#define MAIN_MICROMEGAS
	PROGRAM MAIN

*  Program to compute the NMSSM Higgs masses, couplings and
*  Branching ratios, with experimental and theoretical constraints
*
*   On input:	
*
*	OMGFLAG	Computation of relic density: 0=no, 1=yes
*
*	PAR(1) = lambda
*	PAR(2) = kappa
*	PAR(3) = tan(beta)
*	PAR(4) = mu (effective mu term = lambda*s)
*	PAR(5) = Alambda
*	PAR(6) = Akappa
*	PAR(7) = mQ3**2
*	PAR(8) = mU3**2
*	PAR(9) = mD3**2
*	PAR(10) = mL3**2
*	PAR(11) = mE3**2
*	PAR(12) = AU3
*	PAR(13) = AD3
*	PAR(14) = AE3
*	PAR(15) = mQ**2
*	PAR(16) = mU**2
*	PAR(17) = mD**2
*	PAR(18) = mL**2
*	PAR(19) = mE**2
*	PAR(20) = M1
*	PAR(21) = M2
*	PAR(22) = M3
*	PAR(23) = MA
*       PAR(24) = AE2
*
*	All these parameters are assumed to be defined in DRbar at the scale Q2
*	Q2 is either defined by the user in the input file or computed as Q2 = mQ3*mU3
*
*   On output:
*
*	SMASS(1-3): CP-even masses (ordered)
*
*	SCOMP(1-3,1-3): Mixing angles: if HB(I) are the bare states,
*	  HB(I) = Re(H1), Re(H2), Re(S), and HM(I) are the mass eigenstates,
*	  the convention is HB(I) = SUM_(J=1,3) SCOMP(J,I)*HM(J)
*	  which is equivalent to HM(I) = SUM_(J=1,3) SCOMP(I,J)*HB(J)
*
*	PMASS(1-2): CP-odd masses (ordered)
*
*	PCOMP(1-2,1-2): Mixing angles: if AB(I) are the bare states,
*	  AB(I) = Im(H1), Im(H2), Im(S), and AM(I) are the mass eigenstates,
*	  the convention is
*	  AM(I) = PCOMP(I,1)*(COSBETA*AB(1)+SINBETA*AB(2))
*			+ PCOMP(I,2)*AB(3)
*
*	CMASS: Charged Higgs mass
*
*	CU,CD,CV,CJ,CG(i)	Reduced couplings of h1,h2,h3 (i=1,2,3) or
*				a1,a2 (i=4,5) to up type fermions, down type
*				fermions, gauge bosons, gluons and photons
*				Note: CV(4)=CV(5)=0
*
*	WIDTH(i) Total decay width of h1,h2,h3,a1,a2 (i=1..5)
*		with the following branching ratios:
*	BRJJ(i)  h1,h2,h3,a1,a2	-> gluon gluon
*	BRMM(i)		"	-> mu mu
*	BRLL(i)		"	-> tau tau
*	BRSS(i)		"	-> ss
*	BRCC(i)		"	-> cc
*	BRBB(i)		"	-> bb
*	BRTT(i)		"	-> tt
*	BRWW(i)		"	-> WW (BRWW(4)=BRWW(5)=0)
*	BRZZ(i)		"	-> ZZ (BRZZ(4)=BRZZ(5)=0)
*	BRGG(i)		"	-> gamma gamma
*	BRZG(i)		"	-> Z gamma
*	BRHIGGS(i) (i=1..5) 	-> other Higgses, including:
*		BRHAA(i,j)	hi -> a1a1, a1a2, a2a2 (i=1..3, j=1..3)
*		BRHCHC(i)	hi -> h+h- (i=1..3)
*		BRHAZ(i,j)	hi -> Zaj  (i=1..3)
*		BRHCW(i)	h1,h2,h3 -> h+W- (i=1..3), a1,a2 -> h+W- (i=4,5)
*		BRHHH(i)	H2 -> H1H1, h3-> H1H1, H1H2, H2H2 (i=1..4)
*		BRAHA(i)	a2 -> a1hi (i=1..3)
*		BRAHZ(i,j)	ai -> Zhj  (i=1,2, j=1..3)
*	BRSUSY(i)  (i=1..5)	-> susy particles, including:
*		BRNEU(i,j,k)	-> neutralinos j,k (i=1..5, j,k=1..5)
*		BRCHA(i,j)	-> charginos 11, 12, 22 (i=1..5, j=1..3)
*		BRHSQ(i,j)	hi -> uLuL, uRuR, dLdL, dRdR, t1t1, t2t2,
*				   t1t2, b1b1, b2b2, b1b2 (i=1..3, j=1..10)
*		BRASQ(i,j)	ai -> t1t2, b1b2 (i=1,2, j=1,2)
*		BRHSL(i,j)	hi -> lLlL, lRlR, nLnL, l1l1, l2l2, l1l2,
*				   ntnt (i=1..3, j=1..7)
*		BRASL(i)	ai -> l1l2 (i=1,2)
*
*	HCWIDTH		Total decay width of the charged Higgs
*			with the following branching ratios:
*	HCBRM	     h+	-> mu nu_mu
*	HCBRL		-> tau nu_tau
*	HCBRSU		-> s u
*	HCBRBU		-> b u
*	HCBRSC		-> s c
*	HCBRBC		-> b c
*	HCBRBT		-> b t
*	HCBRWHT		-> neutral Higgs W+, including:
*	     HCBRWH(i)	-> H1W+, H2W+, h3W+, a1W+, a2W+ (i=1..5)
*	HCBRSUSY	-> susy particles,including
*	     HCBRNC(i,j)-> neutralino i chargino j (i=1..5, j=1,2)
*	     HCBRSQ(i)	-> uLdL, t1b1, t1b2, t2b1, t2b2 (i=1..5)
*	     HCBRSL(i)	-> lLnL, t1nt, t2nt (i=1..3)
*
*	MNEU(i)	 Mass of neutralino chi_i (i=1,5, ordered in mass)
*	NEU(i,j) chi_i components of bino, wino, higgsino u&d, singlino
*                (i,j=1..5)
*
*	MCHA(i)		Chargino masses
*	U(i,j),V(i,j)	Chargino mixing matrices
*
*       ERRORS: IFAIL = 0..12
*
*	IFAIL =	0		OK
*		1,3,5,7		m_S1^2 < 0
*		2,3,6,7		m_P1^2 < 0
*		4,5,6,7		m_Hc^2 < 0
*		8		m_sfermion^2 < 0
*		9		l, k, tan(beta) or mu = 0
*		10		Violation of at least one phenomenological constraint
*		11,12		Problem in integration of RGEs
*
*       Phenomenological constraints:
*
*		PROB(I) = 0, I = 1..36: OK
*		
*		PROB(1) =/= 0	chargino too light
*		PROB(2) =/= 0	excluded by Z -> neutralinos
*		PROB(3) =/= 0	charged Higgs too light
*		PROB(4) =/= 0	excluded by ee -> hZ 
*		PROB(5) =/= 0	excluded by ee -> hZ, h -> bb
*		PROB(6) =/= 0	excluded by ee -> hZ, h -> tautau
*		PROB(7) =/= 0	excluded by ee -> hZ, h -> invisible 
*		PROB(8) =/= 0	excluded by ee -> hZ, h -> 2jets
*		PROB(9) =/= 0	excluded by ee -> hZ, h -> 2photons
*		PROB(10) =/= 0	excluded by ee -> hZ, h -> AA -> 4bs
*		PROB(11) =/= 0	excluded by ee -> hZ, h -> AA -> 4taus
*		PROB(12) =/= 0	excluded by ee -> hZ, h -> AA -> 2bs 2taus
*		PROB(13) =/= 0	excluded by Z -> hA (Z width)
*		PROB(14) =/= 0	excluded by ee -> hA -> 4bs
*		PROB(15) =/= 0	excluded by ee -> hA -> 4taus
*		PROB(16) =/= 0	excluded by ee -> hA -> 2bs 2taus
*		PROB(17) =/= 0	excluded by ee -> hA -> AAA -> 6bs
*		PROB(18) =/= 0	excluded by ee -> hA -> AAA -> 6taus
*		PROB(19) =/= 0	excluded by ee -> Zh -> ZAA -> Z + light pairs
*		PROB(20) =/= 0	excluded by stop -> b l sneutrino
*		PROB(21) =/= 0	excluded by stop -> neutralino c
*		PROB(22) =/= 0	excluded by sbottom -> neutralino b
*		PROB(23) =/= 0	squark/gluino too light
*		PROB(24) =/= 0	selectron/smuon too light
*		PROB(25) =/= 0	stau too light
*		PROB(26) =/= 0	lightest neutralino is not LSP
*		PROB(27) =/= 0	Landau Pole in lambda, kappa, ht, hb below MGUT
*		PROB(28) =/= 0	unphysical global minimum
*		PROB(29) =/= 0	Higgs soft masses >> Msusy
*		PROB(30) =/= 0	excluded by WMAP (checked only if OMGFLAG=1)
*		PROB(31) =/= 0	effective Higgs self-couplings in MO > 1
*		PROB(32) =/= 0	b->s gamma more than 2 sigma away
*		PROB(33) =/= 0	Delta M_s more than 2 sigma away
*		PROB(34) =/= 0	Delta M_d more than 2 sigma away
*		PROB(35) =/= 0	B_s->mu+mu- more than 2 sigma away
*		PROB(36) =/= 0	B+-> tau+nu_tau more than 2 sigma away
*
************************************************************************

	IMPLICIT NONE

	INTEGER NPROB,NPAR
	PARAMETER (NPROB=36,NPAR=24)
	INTEGER IFAIL,I,OMGFLAG,MAFLAG

	DOUBLE PRECISION PAR(NPAR),PROB(NPROB)

	COMMON/FLAGS/OMGFLAG,MAFLAG

*   I/O files

	OPEN(15,FILE='inp.dat',STATUS='UNKNOWN')
	OPEN(17,FILE='spectr.dat', STATUS= 'UNKNOWN')
	OPEN(18,FILE='decay.dat', STATUS= 'UNKNOWN')
	OPEN(19,FILE='omega.dat',STATUS='UNKNOWN')

*   Initialization

	CALL INITIALIZE()

*   Reading of the input parameters

	CALL INPUT(PAR,NPAR)

*   Initialization of PROB and IFAIL

	DO I=1,NPROB
	 PROB(I)=0.D0
	ENDDO	
	IFAIL=0

*   Check for singular parameters l, k, tan(beta) and mu_eff

	IF(PAR(1)*PAR(2)*PAR(3)*PAR(4).EQ.0.D0)THEN
	 IFAIL=9
	 GOTO 11
	ENDIF
	
*   Computation of parameters at QSTSB
	
	CALL RUNPAR(PAR)
	
*   Computation of sfermion masses

	CALL MSFERM(PAR,IFAIL)
	IF(IFAIL.NE.0)GOTO 11
	
*   Computation of Higgs masses

	CALL MHIGGS(PAR,IFAIL)
	IF(IFAIL.NE.0)GOTO 11

*   Computation of gluino mass

	CALL GLUINO(PAR)	
	
*   Computation of chargino masses

	CALL CHARGINO(PAR)

*   Computation of neutralino masses

	CALL NEUTRALINO(PAR)

*   Computation of Higgs branching ratios

	CALL DECAY(PAR)

*   Exp. constraints on sparticles/Higgses

	CALL SUBEXP(PAR,PROB)

*   b -> s gamma + B physics

	CALL BSG(PAR,PROB)

*   Global minimum?

	CALL CHECKMIN(PAR,PROB)
	
*   Landau Pole?

	CALL RGES(PAR,PROB,IFAIL)

*   RGEs for the soft terms

	CALL RGESOFT(PAR,IFAIL)

*   Relic density
#ifdef MAIN_NMSSMTOOLS
	CALL RELDEN(PAR,PROB)
#endif
*   Phenomenological problem

	DO I=1,NPROB
	 IF(PROB(I).NE.0.D0 .AND. I.NE.31)IFAIL=10
	ENDDO

*   Recording of the results

 11	CALL OUTPUT(PAR,PROB,IFAIL)

	END


	SUBROUTINE INPUT(PAR,NPAR)

*******************************************************************
*   This subroutine reads from inp.dat (edited in SLHA format)
*   SM and NMSSM parameters.
*******************************************************************

	IMPLICIT NONE

	CHARACTER CHINL*120,CHBLCK*60,CHDUM*120

	INTEGER I,NLINE,INL,ICH,IX,IVAL,INMSSM,Q2FIX
	INTEGER N0,NLOOP,NBER,NPAR,OMGFLAG,MAFLAG

	DOUBLE PRECISION PAR(*),VAL
	DOUBLE PRECISION ACC,XITLA,XLAMBDA,MC0,MB0,MT0
	DOUBLE PRECISION ALSMZ,ALEMMZ,GF,g1,g2,S2TW
	DOUBLE PRECISION MS,MC,MB,MBP,MT,MTAU,MMUON,MZ,MW
	DOUBLE PRECISION VUS,VCB,VUB,Q2,Q2MIN

	COMMON/FLAGS/OMGFLAG,MAFLAG
	COMMON/GAUGE/ALSMZ,ALEMMZ,GF,g1,g2,S2TW
	COMMON/SMSPEC/MS,MC,MB,MBP,MT,MTAU,MMUON,MZ,MW
	COMMON/CKM/VUS,VCB,VUB
	COMMON/ALS/XLAMBDA,MC0,MB0,MT0,N0
	COMMON/RENSCALE/Q2
	COMMON/Q2FIX/Q2MIN,Q2FIX

*   INITIALIZATION OF THE SUSY PARAMETERS
	DO I=1,NPAR
	 PAR(I)=0.D0
	ENDDO	

*   DEFAULT VALUE FOR INMSSM (DEFAULT: 0)
	INMSSM=0

*   DEFAULT VALUE FOR OMGFLAG (DEFAULT: 0)
	OMGFLAG=0

*   DEFAULT VALUE FOR THE RENSCALE Q2 (DEFAULT: 0.D0)
	Q2=0.D0

*   DEFAULT VALUE FOR MAFLAG (DEFAULT: -1)
	MAFLAG=-1

*   INITIALIZE READ LOOP
	NLINE=0
	CHBLCK=' '

*   START TO READ NEW LINE INTO CHINL
 21	CHINL=' '

*   LINE NUMBER
	NLINE=NLINE+1

	READ(15,'(A120)',END=29,ERR=999) CHINL
	
*   CHECK FOR EMPTY OR COMMENT LINES
	IF(CHINL.EQ.' '.OR.CHINL(1:1).EQ.'#'
     C   .OR.CHINL(1:1).EQ.'*') GOTO 21

*   FORCE UPPER CASE LETTERS IN CHINL (AS REQUIRED BELOW)
	INL=0
 22	INL=INL+1
	IF(CHINL(INL:INL).NE.'#') THEN
	 DO ICH=97,122
	  IF(CHINL(INL:INL).EQ.CHAR(ICH)) CHINL(INL:INL)=CHAR(ICH-32)	
	 ENDDO
	 IF(INL.LT.120) GOTO 22
	ENDIF

*   CHECK FOR BLOCK STATEMENT
	IF(CHINL(1:1).EQ.'B') THEN
	 READ(CHINL,'(A6,A)',ERR=999) CHDUM,CHBLCK
	 GOTO 21
	ENDIF

*   CHECK FOR NMSSM MODEL IN MODSEL
*   IF THE RELIC DENSITY SHOULD BE COMPUTED
*   THE BLOCK MODSEL MUST CONTAIN THE LINE "  9     1    "
	IF(CHBLCK(1:6).EQ.'MODSEL') THEN
	 READ(CHINL,*,ERR=999) IX,IVAL
	 IF(IX.EQ.3) INMSSM=IVAL
	 IF(IX.EQ.9) OMGFLAG=IVAL

*   READ SMINPUTS
	ELSEIF(CHBLCK(1:8).EQ.'SMINPUTS') THEN
	 READ(CHINL,*,ERR=999) IX,VAL
	 IF(IX.EQ.2) GF=VAL
	 IF(IX.EQ.3) ALSMZ=VAL
	 IF(IX.EQ.4) MZ=VAL
	 IF(IX.EQ.5) MB=VAL
	 IF(IX.EQ.6) MT=VAL
	 IF(IX.EQ.7) MTAU=VAL

*   READ Q2 AND TANBETA
	ELSEIF(CHBLCK(1:6).EQ.'MINPAR') THEN
	 READ(CHINL,*,ERR=999) IX,VAL
	 IF(IX.EQ.0) Q2=VAL**2
	 IF(IX.EQ.3) PAR(3)=VAL

*   READ EXTPAR
	ELSEIF(CHBLCK(1:6).EQ.'EXTPAR') THEN
	 READ(CHINL,*,ERR=999) IX,VAL
	 IF(IX.EQ.124) THEN
	    IF(MAFLAG.EQ.0)GOTO 99
	    PAR(23)=VAL
	    MAFLAG=1
	 ENDIF
	 IF(IX.EQ.61) PAR(1)=VAL
	 IF(IX.EQ.62) PAR(2)=VAL
	 IF(IX.EQ.63)THEN
	    IF(MAFLAG.EQ.1)GOTO 99
	    PAR(5)=VAL
	    MAFLAG=0
	 ENDIF
	 IF(IX.EQ.64) PAR(6)=VAL
	 IF(IX.EQ.65) PAR(4)=VAL
	 IF(IX.EQ.1) PAR(20)=VAL
	 IF(IX.EQ.2) PAR(21)=VAL
	 IF(IX.EQ.3) PAR(22)=VAL
	 IF(IX.EQ.11) PAR(12)=VAL
	 IF(IX.EQ.12) PAR(13)=VAL
	 IF(IX.EQ.13) PAR(14)=VAL
	 IF(IX.EQ.16) PAR(24)=VAL
	 IF(IX.EQ.31) PAR(18)=VAL**2
	 IF(IX.EQ.32) PAR(18)=VAL**2
	 IF(IX.EQ.33) PAR(10)=VAL**2
	 IF(IX.EQ.34) PAR(19)=VAL**2
	 IF(IX.EQ.35) PAR(19)=VAL**2
	 IF(IX.EQ.36) PAR(11)=VAL**2
	 IF(IX.EQ.41) PAR(15)=VAL**2
	 IF(IX.EQ.42) PAR(15)=VAL**2
	 IF(IX.EQ.43) PAR(7)=VAL**2
	 IF(IX.EQ.44) PAR(16)=VAL**2
	 IF(IX.EQ.45) PAR(16)=VAL**2
	 IF(IX.EQ.46) PAR(8)=VAL**2
	 IF(IX.EQ.47) PAR(17)=VAL**2
	 IF(IX.EQ.48) PAR(17)=VAL**2
	 IF(IX.EQ.49) PAR(9)=VAL**2

	ENDIF

	GOTO 21

*   END OF READING FROM INPUT FILE
 29	IF(INMSSM.NE.1) THEN
	 WRITE(*,*) 'SWITCH IN MODSEL DOES NOT CORRESPOND TO THE NMSSM'
         STOP
	ENDIF

	IF(MAFLAG.EQ.-1)THEN
	 WRITE(*,*)'Error in input file inp.dat:'
	 WRITE(*,*)'Either Alambda or MA has to be fixed'
	 STOP
	ENDIF

*   Set Q2MIN, Q2FIX:
	Q2MIN=100.D0**2
	Q2FIX=1
	IF(Q2.LE.Q2MIN) THEN
	 Q2FIX=0
	ENDIF

*   Initialization for ALPHAS and RUNM (as in hdecay)
*   The bottom quark pole mass MBP is set in INIT and can be changed
*   only there (changing its running mass MB above has no effect
*   on MBP, since one would have to compute alpha_s(MB) first)

	MC0=MC
	MB0=MBP
	MT0=MT
	N0=5
	NLOOP=2
	NBER=18
	ACC=1.D-8
	XLAMBDA=XITLA(NLOOP,ALSMZ,ACC)
	CALL ALSINI(ACC)
	CALL BERNINI(NBER)

*    g1,g2  and sin(theta)^2 in the on-shell scheme in terms of
*    GF, MZ(pole) and MW(pole)

	g2=4.D0*DSQRT(2.D0)*GF*MW**2
	g1=4.D0*DSQRT(2.D0)*GF*(MZ**2-MW**2)
	S2TW=1.D0-(MW/MZ)**2

	RETURN

 99	WRITE(*,*)'Error in input file inp.dat:'
	WRITE(*,*)'Alambda and MA cannot be fixed at the same time'
	STOP

 999    WRITE(*,*) 'inp.dat READ ERROR ON LINE:', NLINE
        WRITE(*,*) CHINL(1:80)
	STOP

	END


	SUBROUTINE OUTPUT(PAR,PROB,IFAIL)

	IMPLICIT NONE

	INTEGER I,NBIN,IFAIL,OMGFLAG,MAFLAG

	DOUBLE PRECISION PAR(*),PROB(*)
	DOUBLE PRECISION SMASS(3),SCOMP(3,3),PMASS(2),PCOMP(2,2),CMASS
	DOUBLE PRECISION MGL,MCHA(2),U(2,2),V(2,2),MNEU(5),NEU(5,5)
	DOUBLE PRECISION BRJJ(5),BRMM(5),BRLL(5),BRSS(5),BRCC(5)
	DOUBLE PRECISION BRBB(5),BRTT(5),BRWW(3),BRZZ(3),BRGG(5)
	DOUBLE PRECISION BRZG(5),BRHHH(4),BRHAA(3,3),BRHCHC(3)
	DOUBLE PRECISION BRHAZ(3,2),BRAHA(3),BRAHZ(2,3),BRHCW(5)
	DOUBLE PRECISION BRHIGGS(5),BRNEU(5,5,5),BRCHA(5,3)
	DOUBLE PRECISION BRHSQ(3,10),BRHSL(3,7),BRASQ(2,6),BRASL(2,3)
	DOUBLE PRECISION BRSUSY(5),WIDTH(5)
	DOUBLE PRECISION HCBRM,HCBRL,HCBRSU,HCBRBU,HCBRSC,HCBRBC
	DOUBLE PRECISION HCBRBT,HCBRWH(5),HCBRWHT,HCBRNC(5,2)
	DOUBLE PRECISION HCBRSQ(5),HCBRSL(3),HCBRSUSY,HCWIDTH
	DOUBLE PRECISION CU(5),CD(5),CV(3),CJ(5),CG(5)
	DOUBLE PRECISION ALSMZ,ALEMMZ,GF,g1,g2,S2TW
	DOUBLE PRECISION MS,MC,MB,MBP,MT,MTAU,MMUON,MZ,MW
	DOUBLE PRECISION VUS,VCB,VUB,TANB,SINB,COSB
	DOUBLE PRECISION MUR,MUL,MDR,MDL,MLR,MLL,MNL
	DOUBLE PRECISION MST1,MST2,MSB1,MSB2,MSL1,MSL2,MSNT
	DOUBLE PRECISION CST,CSB,CSL,MSMU1,MSMU2,MSMUNT,CSMU
	DOUBLE PRECISION SST,SSB,SSL,Q2
	DOUBLE PRECISION MGUT,g1s,g2s,g3s,HTOPS,HBOTS,HTAUS
	DOUBLE PRECISION MHUS,MHDS,MSSS
	DOUBLE PRECISION G1GUT,G2GUT,G3GUT,LGUT,KGUT,HTOPGUT
	DOUBLE PRECISION HBOTGUT,HTAUGUT,M1GUT,M2GUT,M3GUT,ALGUT,AKGUT
	DOUBLE PRECISION ATGUT,ABGUT,ATAUGUT,AMUGUT,MHUGUT,MHDGUT
	DOUBLE PRECISION MSSGUT,MQ3GUT
	DOUBLE PRECISION MU3GUT,MD3GUT,MQGUT,MUGUT,MDGUT,ML3GUT,ME3GUT
	DOUBLE PRECISION MLGUT,MEGUT
	DOUBLE PRECISION OMG,OMGMIN,OMGMAX,LOPMASS
	DOUBLE PRECISION Xf,sigmaV,x(100),dNdx(100),EMIN
	DOUBLE PRECISION MHH,MAA,MSS,MHA,MHS,MAS,MPP,MPS,MPM
	DOUBLE PRECISION MH1X,MH2X,MSSX,LQ,KQ,ALQ,AKQ,MUQ,NUQ
	DOUBLE PRECISION ZHU,ZHD,ZS,H1Q,H2Q,TANBQ,QSTSB
	DOUBLE PRECISION BRSG,BRSGmax,BRSGmin,DMd,DMdmin,DMdmax,DMs,
     C        DMsmax,DMsmin,BRBMUMU,BRBMUMUmax,BRBMUMUmin,BRBtaunu,
     C        BRBtaunumax,BRBtaunumin
	

	COMMON/BRN/BRJJ,BRMM,BRLL,BRSS,BRCC,BRBB,BRTT,BRWW,BRZZ,
     C		BRGG,BRZG,BRHHH,BRHAA,BRHCHC,BRHAZ,BRAHA,BRAHZ,
     C		BRHCW,BRHIGGS,BRNEU,BRCHA,BRHSQ,BRHSL,BRASQ,BRASL,
     C		BRSUSY,WIDTH
	COMMON/BRC/HCBRM,HCBRL,HCBRSU,HCBRBU,HCBRSC,HCBRBC,
     C		HCBRBT,HCBRWH,HCBRWHT,HCBRNC,HCBRSQ,HCBRSL,
     C		HCBRSUSY,HCWIDTH
	COMMON/REDCOUP/CU,CD,CV,CJ,CG
	COMMON/HIGGSPEC/SMASS,SCOMP,PMASS,PCOMP,CMASS
	COMMON/SUSYSPEC/MGL,MCHA,U,V,MNEU,NEU
	COMMON/GAUGE/ALSMZ,ALEMMZ,GF,g1,g2,S2TW
	COMMON/SMSPEC/MS,MC,MB,MBP,MT,MTAU,MMUON,MZ,MW
	COMMON/CKM/VUS,VCB,VUB
	COMMON/SFSPEC/MUR,MUL,MDR,MDL,MLR,MLL,MNL,
     C		MST1,MST2,MSB1,MSB2,MSL1,MSL2,MSNT,
     C		CST,CSB,CSL,MSMU1,MSMU2,MSMUNT,CSMU
        COMMON/BRSG/BRSG,BRSGmax,BRSGmin,DMd,DMdmin,DMdmax,DMs,
     C        DMsmax,DMsmin,BRBMUMU,BRBMUMUmax,BRBMUMUmin,BRBtaunu,
     C        BRBtaunumax,BRBtaunumin
	COMMON/RENSCALE/Q2
	COMMON/STSBSCALE/QSTSB
	COMMON/SUSYCOUP/MGUT,g1s,g2s,g3s,HTOPS,HBOTS,HTAUS
	COMMON/SUSYMH/MHUS,MHDS,MSSS
	COMMON/QMHIGGS/MH1X,MH2X,MSSX
	COMMON/QNMPAR/LQ,KQ,ALQ,AKQ,MUQ,NUQ
	COMMON/QHIGGS/ZHU,ZHD,ZS,H1Q,H2Q,TANBQ
	COMMON/GUTCOUP/G1GUT,G2GUT,G3GUT,LGUT,KGUT,HTOPGUT,
     C        HBOTGUT,HTAUGUT
	COMMON/GUTPAR/M1GUT,M2GUT,M3GUT,ALGUT,AKGUT,ATGUT,ABGUT,
     C        ATAUGUT,AMUGUT,MHUGUT,MHDGUT,MSSGUT,MQ3GUT,MU3GUT,MD3GUT,
     C        MQGUT,MUGUT,MDGUT,ML3GUT,ME3GUT,MLGUT,MEGUT
	COMMON/MICROMG/OMG,OMGMIN,OMGMAX,Xf,sigmaV,x,dNdx,EMIN,NBIN
	COMMON/FLAGS/OMGFLAG,MAFLAG
	COMMON/HOUT/MHH,MAA,MSS,MHA,MHS,MAS,MPP,MPS,MPM
#ifdef  MAIN_MICROMEGAS
        DOUBLE PRECISION DELMB,MH(3),MA(2),MHC  
        COMMON/DELMB/DELMB
        COMMON/HMO/MH,MA,MHC
#endif

	TANB=PAR(3)
	COSB=1.D0/DSQRT(1.D0+TANB**2)
	SINB=TANB*COSB

	WRITE(17,899) "# NMHDECAY OUTPUT IN SLHA FORMAT"
	WRITE(17,899) "# Info about spectrum calculator"
	WRITE(17,899) "BLOCK SPINFO        # Program information"
	WRITE(17,900) 1,"NMHDECAY   # spectrum calculator"
	WRITE(17,900) 2,"3          # version number"
	
	IF(PROB(1).NE.0.D0)
     C   WRITE(17,900) 3,"# Chargino too light"
	IF(PROB(2).NE.0.D0)
     C   WRITE(17,900) 3,"# Neutralinos too light"
        IF(PROB(3).NE.0.D0)
     C   WRITE(17,900) 3,"# Charged Higgs too light"
        IF(PROB(4).NE.0.D0)
     C   WRITE(17,900) 3,"# Excluded by ee -> hZ, ind. of h decay"
        IF(PROB(5).NE.0.D0)
     C   WRITE(17,900) 3,"# Excluded by ee -> hZ, h -> bb"
        IF(PROB(6).NE.0.D0)
     C   WRITE(17,900) 3,"# Excluded by ee -> hZ, h -> tautau"
        IF(PROB(7).NE.0.D0)
     C   WRITE(17,900) 3,"# Excluded by ee -> hZ, h -> invisible"
        IF(PROB(8).NE.0.D0)
     C   WRITE(17,900) 3,"# Excluded by ee -> hZ, h -> 2jets"
        IF(PROB(9).NE.0.D0)
     C   WRITE(17,900) 3,"# Excluded by ee -> hZ, h -> 2photons"
        IF(PROB(10).NE.0.D0)
     C   WRITE(17,900) 3,"# Excluded by ee -> hZ, h -> AA -> 4bs"
        IF(PROB(11).NE.0.D0)
     C   WRITE(17,900) 3,"# Excluded by ee -> hZ, h -> AA -> 4taus"
        IF(PROB(12).NE.0.D0)
     C   WRITE(17,900) 3,"# Excluded by ee -> hZ, h -> AA -> 2bs 2taus"
        IF(PROB(19).NE.0.D0)
     C   WRITE(17,900) 3,
     C	  "# Excluded by ee -> hZ, h -> AA,A -> light pair"
        IF(PROB(13).NE.0.D0)
     C   WRITE(17,900) 3,"# Excluded by ee -> Z -> hA (Z width)"
        IF(PROB(14).NE.0.D0)
     C   WRITE(17,900) 3,"# Excluded by ee -> hA -> 4bs"
        IF(PROB(15).NE.0.D0)
     C   WRITE(17,900) 3,"# Excluded by ee -> hA -> 4taus"
        IF(PROB(16).NE.0.D0)
     C   WRITE(17,900) 3,"# Excluded by ee -> hA -> 2bs 2taus"
        IF(PROB(17).NE.0.D0)
     C   WRITE(17,900) 3,"# Excluded by ee -> hA -> AAA -> 6bs"
        IF(PROB(18).NE.0.D0)
     C   WRITE(17,900) 3,"# Excluded by ee -> hA -> AAA -> 6taus"
        IF(PROB(20).NE.0.D0)
     C   WRITE(17,900) 3,"# Excluded by stop -> b l sneutrino"
        IF(PROB(21).NE.0.D0)
     C   WRITE(17,900) 3,"# Excluded by stop -> neutralino c"
        IF(PROB(22).NE.0.D0)
     C   WRITE(17,900) 3,"# Excluded by sbottom -> neutralino b"
        IF(PROB(23).NE.0.D0)
     C   WRITE(17,900) 3,"# Squark/gluino too light"
        IF(PROB(24).NE.0.D0)
     C   WRITE(17,900) 3,"# Selectron/smuon too light"
        IF(PROB(25).NE.0.D0)
     C   WRITE(17,900) 3,"# Stau too light"
        IF(PROB(26).NE.0.D0)
     C   WRITE(17,900) 3,"# Lightest neutralino is not the LSP"
        IF(PROB(27).NE.0.D0)
     C   WRITE(17,900) 3,"# Landau Pole below MGUT"
        IF(PROB(28).NE.0.D0)
     C   WRITE(17,900) 3,"# Unphysical global minimum"
        IF(PROB(29).NE.0.D0)
     C   WRITE(17,900) 3,"# Higgs soft masses >> Msusy"
#ifdef  MAIN_NMSSMTOOLS
	IF(OMGFLAG.NE.0 .AND. OMG.GT.0.D0)THEN
          IF(PROB(30).GT.0.D0)WRITE(17,900) 3,
     C      "# Relic density too large (WMAP)"
          IF(PROB(30).LT.0.D0)WRITE(17,900) 3,
     C      "# Relic density too small (WMAP)"
	ENDIF
#endif
        IF(PROB(32).NE.0.D0)
     C   WRITE(17,900) 3,"# b->s gamma more than 2 sigma away"
        IF(PROB(33).NE.0.D0)
     C   WRITE(17,900) 3,"# Delta M_s more than 2 sigma away"
        IF(PROB(34).NE.0.D0)
     C   WRITE(17,900) 3,"# Delta M_d more than 2 sigma away"
        IF(PROB(35).NE.0.D0)
     C   WRITE(17,900) 3,"# B_s->mu+mu- more than 2 sigma away"
        IF(PROB(36).NE.0.D0)
     C   WRITE(17,900) 3,"# B+-> tau+nu_tau more than 2 sigma away"

	IF(IFAIL.EQ.1 .OR. IFAIL.EQ.3 .OR. IFAIL.EQ.5 .OR. IFAIL.EQ.7)
     C	   WRITE(17,900) 4,"# M_H1^2<0"
	IF(IFAIL.EQ.2 .OR. IFAIL.EQ.3 .OR. IFAIL.EQ.6 .OR. IFAIL.EQ.7)
     C	   WRITE(17,900) 4,"# M_A1^2<0"
	IF(IFAIL.EQ.4 .OR. IFAIL.EQ.5 .OR. IFAIL.EQ.6 .OR. IFAIL.EQ.7)
     C	   WRITE(17,900) 4,"# M_HC^2<0"
	IF (IFAIL.EQ.8)
     C 	   WRITE(17,900) 4,"# Negative sfermion mass squared"
	IF (IFAIL.EQ.9)
     C 	   WRITE(17,900) 4,"# Disallowed parameters: lambda, kappa,
     C tan(beta) or mu=0"
	IF (IFAIL.EQ.11)
     C 	   WRITE(17,900) 4,"# Integration problem in RGES"
	IF (IFAIL.EQ.12)
     C 	   WRITE(17,900) 4,"# Integration problem in RGESOFT"

	WRITE(17,899) "# Input parameters"
	WRITE(17,899) "BLOCK MODSEL"
	WRITE(17,899) "    3     1         # NMSSM PARTICLE CONTENT"
	WRITE(17,899) "BLOCK SMINPUTS"
	WRITE(17,901) 1,1.D0/ALEMMZ,"ALPHA_EM^-1(MZ)"
	WRITE(17,901) 2,GF,"GF"
	WRITE(17,901) 3,ALSMZ,"ALPHA_S(MZ)"
	WRITE(17,901) 4,MZ,"MZ"
	WRITE(17,901) 5,MB,"MB(MB)"
	WRITE(17,901) 6,MT,"MTOP (POLE MASS)"
	WRITE(17,901) 7,MTAU,"MTAU"
	WRITE(17,899) "# SMINPUTS Beyond SLHA:"
	WRITE(17,906) "MW:",MW
	WRITE(17,906) "MS:",MS
	WRITE(17,906) "MC:",MC
	WRITE(17,906) "VUS:",VUS
	WRITE(17,906) "VCB:",VCB
	WRITE(17,906) "VUB:",VUB

	WRITE(17,899) "BLOCK MINPAR"
	WRITE(17,901) 3,TANB,"TANBETA"

	WRITE(17,899) "BLOCK EXTPAR"
	WRITE(17,901) 1,PAR(20),"M1"
	WRITE(17,901) 2,PAR(21),"M2"
	WRITE(17,901) 3,PAR(22),"M3"
	WRITE(17,901) 11,PAR(12),"ATOP"
	WRITE(17,901) 12,PAR(13),"ABOTTOM"
	WRITE(17,901) 13,PAR(14),"ATAU"
	WRITE(17,901) 16,PAR(24),"AMUON"
	WRITE(17,901) 124,PAR(23),"MA AT Q_STSB"
	
	WRITE(17,901) 31,DSQRT(PAR(18)),"LEFT SELECTRON"
	WRITE(17,901) 32,DSQRT(PAR(18)),"LEFT SMUON"
	WRITE(17,901) 33,DSQRT(PAR(10)),"LEFT STAU"
	
	WRITE(17,901) 34,DSQRT(PAR(19)),"RIGHT SELECTRON"
	WRITE(17,901) 35,DSQRT(PAR(19)),"RIGHT SMUON"
	WRITE(17,901) 36,DSQRT(PAR(11)),"RIGHT STAU"
	
	WRITE(17,901) 41,DSQRT(PAR(15)),"LEFT 1ST GEN. SQUARKS"
	WRITE(17,901) 42,DSQRT(PAR(15)),"LEFT 2ND GEN. SQUARKS"
	WRITE(17,901) 43,DSQRT(PAR(7)),"LEFT 3RD GEN. SQUARKS"
	
	WRITE(17,901) 44,DSQRT(PAR(16)),"RIGHT U-SQUARKS"
	WRITE(17,901) 45,DSQRT(PAR(16)),"RIGHT C-SQUARKS"
	WRITE(17,901) 46,DSQRT(PAR(8)),"RIGHT T-SQUARKS"
	
	WRITE(17,901) 47,DSQRT(PAR(17)),"RIGHT D-SQUARKS"
	WRITE(17,901) 48,DSQRT(PAR(17)),"RIGHT S-SQUARKS"
	WRITE(17,901) 49,DSQRT(PAR(9)),"RIGHT B-SQUARKS"
	
	WRITE(17,901) 61,PAR(1),"LAMBDA"
	WRITE(17,901) 62,PAR(2),"KAPPA"
	WRITE(17,901) 63,PAR(5),"A_LAMBDA"
	WRITE(17,901) 64,PAR(6),"A_KAPPA"
	WRITE(17,901) 65,PAR(4),"MU_EFF"

	IF(IFAIL.GT.0 .AND. IFAIL.LT.10) GOTO 1

	WRITE(17,899) "# "
	WRITE(17,899) "BLOCK MASS   # Mass spectrum "
	WRITE(17,899) "#  PDG Code     mass             particle "
	WRITE(17,902) 25,SMASS(1),"lightest neutral scalar"
	WRITE(17,902) 35,SMASS(2),"second neutral scalar"
	WRITE(17,902) 45,SMASS(3),"third neutral scalar"
	WRITE(17,902) 36,PMASS(1),"lightest pseudoscalar"
	WRITE(17,902) 46,PMASS(2),"second pseudoscalar"
	WRITE(17,902) 37,CMASS,"charged Higgs"
	
	WRITE(17,902) 1000001,MDL," ~d_L"
	WRITE(17,902) 2000001,MDR," ~d_R"
	WRITE(17,902) 1000002,MUL," ~u_L"
	WRITE(17,902) 2000002,MUR," ~u_R" 	
	WRITE(17,902) 1000003,MDL," ~s_L"
	WRITE(17,902) 2000003,MDR," ~s_R" 	
	WRITE(17,902) 1000004,MUL," ~c_L"
	WRITE(17,902) 2000004,MUR," ~c_R" 	
	WRITE(17,902) 1000005,MSB1," ~b_1"
	WRITE(17,902) 2000005,MSB2," ~b_2" 	
	WRITE(17,902) 1000006,MST1," ~t_1"
	WRITE(17,902) 2000006,MST2," ~t_2" 	
	WRITE(17,902) 1000011,MLL," ~e_L"
	WRITE(17,902) 2000011,MLR," ~e_R"
	WRITE(17,902) 1000012,MNL," ~nue_L"
	WRITE(17,902) 1000013,MSMU1," ~mu_1"
	WRITE(17,902) 2000013,MSMU2," ~mu_2" 	
	WRITE(17,902) 1000014,MSMUNT," ~numu_L"
	WRITE(17,902) 1000015,MSL1," ~tau_1"
	WRITE(17,902) 2000015,MSL2," ~tau_2"
	WRITE(17,902) 1000016,MSNT," ~nutau_L"
	
	WRITE(17,902) 1000021,MGL," ~g"
	
	WRITE(17,902) 1000022,MNEU(1),"neutralino(1)"
	WRITE(17,902) 1000023,MNEU(2),"neutralino(2)"
	WRITE(17,902) 1000025,MNEU(3),"neutralino(3)"
	WRITE(17,902) 1000035,MNEU(4),"neutralino(4)"
	WRITE(17,902) 1000045,MNEU(5),"neutralino(5)"
	WRITE(17,902) 1000024,MCHA(1),"chargino(1)"
	WRITE(17,902) 1000037,MCHA(2),"chargino(2)"
	
	WRITE(17,899) "# " 
	WRITE(17,899) "# Low energy observables"
	WRITE(17,899) "BLOCK LOWEN"
	WRITE(17,899) 
     C   "# Exp. 2 Sigma: 3.07E-04 < BR(b -> s gamma) < 4.07E-04:"
	WRITE(17,902) 1,BRSG,"BR(b -> s gamma)"
	WRITE(17,902) 11,BRSGMAX,"(BR(b -> s gamma)+Theor.Err.)"
	WRITE(17,902) 12,BRSGMIN,"(BR(b -> s gamma)-Theor.Err.)"
	WRITE(17,899) "# Exp. 2 Sigma: 4.99E-01 < Delta M_d < 5.15E-01:"
	WRITE(17,902) 2,DMD,"Delta M_d in ps^-1"
	WRITE(17,902) 21,DMdmax,"Delta M_d +Theor.Err."
	WRITE(17,902) 22,DMdmin,"Delta M_d -Theor.Err."
	WRITE(17,899) 
     C   "# Exp. 2 Sigma: 1.753E+01 < Delta Ms < 1.801E+01:"
	WRITE(17,902) 3,DMS,"Delta M_s in ps^-1"
	WRITE(17,902) 31,DMsmax,"Delta M_s +Theor.Err."
	WRITE(17,902) 32,DMsmin,"Delta M_s -Theor.Err."
	WRITE(17,899) "# Exp. 95% C.L.: BR(Bs->mu+mu-) < 5.8E-08:"
	WRITE(17,902) 4,BRBMUMU,"BR(Bs -> mu+mu-)"
	WRITE(17,902) 41,BRBMUMUmax,"BR(Bs -> mu+mu-)+Theor.Err."
	WRITE(17,902) 42,BRBMUMUmin,"BR(Bs -> mu+mu-)-Theor.Err."
	WRITE(17,899) 
     C   "# Exp. 2 Sigma: 3.40E-05 < BR(B+ > tau+ + nu_tau) < 2.30E-04:"
	WRITE(17,902) 5,BRBtaunu,"BR(B+ -> tau+ + nu_tau)"
	WRITE(17,902) 51,BRBtaunumax,
     C   "BR(B+ -> tau+ + nu_tau) + Theor.Err."
	WRITE(17,902) 52,BRBtaunumin,
     C   "BR(B+ -> tau+ + nu_tau) - Theor.Err."
#ifdef  MAIN_NMSSMTOOLS
	IF(OMGFLAG.NE.0)THEN
          WRITE(17,911)
     C	   "# Omega h^2 (allowed:",OMGMIN," < Omega h^2 <",OMGMAX,"):"
	  IF(OMG.EQ.0.D0)THEN
	    WRITE(17,899) "# Cannot compute Omega h^2 (mLSP < 1 GeV)"
  	  ELSEIF(OMG.EQ.-1.D0)THEN
	    WRITE(17,899) "# Lightest neutralino is not the LSP"
	  ELSE
	    WRITE(17,902) 10,OMG,"Omega h^2"
	  ENDIF
	ENDIF
#endif	
  	WRITE(17,899) "# " 
        WRITE(17,907) "BLOCK HMIX AT Q=",DSQRT(QSTSB),
     C    " # (STOP/SBOTTOM MASSES)"
	WRITE(17,901) 1,MUQ,"MU_EFF"
	WRITE(17,901) 2,TANBQ,"TAN(BETA)"
	WRITE(17,901) 3,DSQRT(H1Q**2+H2Q**2),"V(Q)"
	WRITE(17,901) 4,PAR(23)**2,"MA**2"	
	
	WRITE(17,899) "# "
	WRITE(17,899) "# 3*3 Higgs mixing"
	WRITE(17,899) "BLOCK NMHMIX"
	WRITE(17,903) 1,1,SCOMP(1,2),"S_(1,1)"
	WRITE(17,903) 1,2,SCOMP(1,1),"S_(1,2)"
	WRITE(17,903) 1,3,SCOMP(1,3),"S_(1,3)"
	WRITE(17,903) 2,1,SCOMP(2,2),"S_(2,1)"
	WRITE(17,903) 2,2,SCOMP(2,1),"S_(2,2)"
	WRITE(17,903) 2,3,SCOMP(2,3),"S_(2,3)"
	WRITE(17,903) 3,1,SCOMP(3,2),"S_(3,1)"
	WRITE(17,903) 3,2,SCOMP(3,1),"S_(3,2)"
	WRITE(17,903) 3,3,SCOMP(3,3),"S_(3,3)"
	
	WRITE(17,899) "# "
	WRITE(17,899) "# 3*3 Pseudoscalar Higgs mixing"
	WRITE(17,899) "BLOCK NMAMIX"
	WRITE(17,903) 1,1,SINB*PCOMP(1,1),"P_(1,1)"
	WRITE(17,903) 1,2,COSB*PCOMP(1,1),"P_(1,2)"
	WRITE(17,903) 1,3,PCOMP(1,2),"P_(1,3)"
	WRITE(17,903) 2,1,SINB*PCOMP(2,1),"P_(2,1)"
	WRITE(17,903) 2,2,COSB*PCOMP(2,1),"P_(2,2)"
	WRITE(17,903) 2,3,PCOMP(2,2),"P_(2,3)"
			
	SST=DSQRT(1-CST**2)
	SSB=DSQRT(1-CSB**2)
	SSL=DSQRT(1-CSL**2)
	
	WRITE(17,899) "# "
	WRITE(17,899) "# 3rd generation sfermion mixing"
	WRITE(17,899) "BLOCK STOPMIX  # Stop mixing matrix"
	WRITE(17,903) 1,1,CST,"Rst_(1,1)"
	WRITE(17,903) 1,2,SST,"Rst_(1,2)"
	WRITE(17,903) 2,1,-SST,"Rst_(2,1)"
	WRITE(17,903) 2,2,CST,"Rst_(2,2)"
	WRITE(17,899) "BLOCK SBOTMIX  # Sbottom mixing matrix"
	WRITE(17,903) 1,1,CSB,"Rsb_(1,1)"
	WRITE(17,903) 1,2,SSB,"Rsb_(1,2)"
	WRITE(17,903) 2,1,-SSB,"Rsb_(2,1)"
	WRITE(17,903) 2,2,CSB,"Rsb_(2,2)"
	WRITE(17,899) "BLOCK STAUMIX  # Stau mixing matrix"
	WRITE(17,903) 1,1,CSL,"Rsl_(1,1)"
	WRITE(17,903) 1,2,SSL,"Rsl_(1,2)"
	WRITE(17,903) 2,1,-SSL,"Rsl_(2,1)"
	WRITE(17,903) 2,2,CSL,"Rsl_(2,2)"

	WRITE(17,899) "# "
	WRITE(17,899) "# Gaugino-Higgsino mixing"
	WRITE(17,899) "BLOCK NMNMIX  # 5*5 Neutralino Mixing Matrix"
	WRITE(17,903) 1,1,NEU(1,1),"N_(1,1)"
	WRITE(17,903) 1,2,NEU(1,2),"N_(1,2)"
	WRITE(17,903) 1,3,NEU(1,4),"N_(1,3)"
	WRITE(17,903) 1,4,NEU(1,3),"N_(1,4)"
	WRITE(17,903) 1,5,NEU(1,5),"N_(1,5)"
	WRITE(17,903) 2,1,NEU(2,1),"N_(2,1)"
	WRITE(17,903) 2,2,NEU(2,2),"N_(2,2)"
	WRITE(17,903) 2,3,NEU(2,4),"N_(2,3)"
	WRITE(17,903) 2,4,NEU(2,3),"N_(2,4)"
	WRITE(17,903) 2,5,NEU(2,5),"N_(2,5)"
	WRITE(17,903) 3,1,NEU(3,1),"N_(3,1)"
	WRITE(17,903) 3,2,NEU(3,2),"N_(3,2)"
	WRITE(17,903) 3,3,NEU(3,4),"N_(3,3)"
	WRITE(17,903) 3,4,NEU(3,3),"N_(3,4)"
	WRITE(17,903) 3,5,NEU(3,5),"N_(3,5)"
	WRITE(17,903) 4,1,NEU(4,1),"N_(4,1)"
	WRITE(17,903) 4,2,NEU(4,2),"N_(4,2)"
	WRITE(17,903) 4,3,NEU(4,4),"N_(4,3)"
	WRITE(17,903) 4,4,NEU(4,3),"N_(4,4)"
	WRITE(17,903) 4,5,NEU(4,5),"N_(4,5)"
	WRITE(17,903) 5,1,NEU(5,1),"N_(5,1)"
	WRITE(17,903) 5,2,NEU(5,2),"N_(5,2)"
	WRITE(17,903) 5,3,NEU(5,4),"N_(5,3)"
	WRITE(17,903) 5,4,NEU(5,3),"N_(5,4)"
	WRITE(17,903) 5,5,NEU(5,5),"N_(5,5)"

	WRITE(17,899) "# "
	WRITE(17,899) "BLOCK UMIX  # Chargino U Mixing Matrix"
	WRITE(17,903) 1,1,U(1,1),"U_(1,1)"
	WRITE(17,903) 1,2,U(1,2),"U_(1,2)"
	WRITE(17,903) 2,1,U(2,1),"U_(2,1)"
	WRITE(17,903) 2,2,U(2,2),"U_(2,2)"

	WRITE(17,899) "# "
	WRITE(17,899) "BLOCK VMIX  # Chargino V Mixing Matrix"
	WRITE(17,903) 1,1,V(1,1),"V_(1,1)"
	WRITE(17,903) 1,2,V(1,2),"V_(1,2)"
	WRITE(17,903) 2,1,V(2,1),"V_(2,1)"
	WRITE(17,903) 2,2,V(2,2),"V_(2,2)"

	WRITE(17,899) "# "
	WRITE(17,899) "# Higgs reduced couplings"
	WRITE(17,899) "# (as compared to a SM Higgs with same mass)"
	WRITE(17,899) "BLOCK REDCOUP"
	WRITE(17,899) "# H1"
	WRITE(17,903) 1,1,CU(1),"U-type fermions"
	WRITE(17,903) 1,2,CD(1),"D-type fermions"
	WRITE(17,903) 1,3,CV(1),"W,Z bosons"
	WRITE(17,903) 1,4,CJ(1),"Gluons"
	WRITE(17,903) 1,5,CG(1),"Photons"
	WRITE(17,899) "# H2"
	WRITE(17,903) 2,1,CU(2),"U-type fermions"
	WRITE(17,903) 2,2,CD(2),"D-type fermions"
	WRITE(17,903) 2,3,CV(2),"W,Z bosons"
	WRITE(17,903) 2,4,CJ(2),"Gluons"
	WRITE(17,903) 2,5,CG(2),"Photons"
	WRITE(17,899) "# H3"
	WRITE(17,903) 3,1,CU(3),"U-type fermions"
	WRITE(17,903) 3,2,CD(3),"D-type fermions"
	WRITE(17,903) 3,3,CV(3),"W,Z bosons"
	WRITE(17,903) 3,4,CJ(3),"Gluons"
	WRITE(17,903) 3,5,CG(3),"Photons"
	WRITE(17,899) "# A1"
	WRITE(17,903) 4,1,CU(4),"U-type fermions"
	WRITE(17,903) 4,2,CD(4),"D-type fermions"
	WRITE(17,903) 4,3,0.,"W,Z bosons"
	WRITE(17,903) 4,4,CJ(4),"Gluons"
	WRITE(17,903) 4,5,CG(4),"Photons"
	WRITE(17,899) "# A2"
	WRITE(17,903) 5,1,CU(5),"U-type fermions"
	WRITE(17,903) 5,2,CD(5),"D-type fermions"
	WRITE(17,903) 5,3,0.,"W,Z bosons"
	WRITE(17,903) 5,4,CJ(5),"Gluons"
	WRITE(17,903) 5,5,CG(5),"Photons"

	WRITE(17,899) "# "
	WRITE(17,899) "# GAUGE AND YUKAWA COUPLINGS AT THE SUSY SCALE"
	WRITE(17,907) "BLOCK GAUGE Q=",DSQRT(Q2)," # (SUSY SCALE)"
	WRITE(17,902) 1,DSQRT(G1S),"g1(Q,DR_bar)"
	WRITE(17,902) 2,DSQRT(G2S),"g2(Q,DR_bar)"
	WRITE(17,902) 3,DSQRT(G3S),"g3(Q,DR_bar)"
	
	WRITE(17,907) "BLOCK YU Q=",DSQRT(Q2)," # (SUSY SCALE)"
	WRITE(17,903) 3,3,HTOPS,"HTOP(Q,DR_bar)"
	WRITE(17,907) "BLOCK YD Q=",DSQRT(Q2)," # (SUSY SCALE)"
	WRITE(17,903) 3,3,HBOTS,"HBOT(Q,DR_bar)"
	WRITE(17,907) "BLOCK YE Q=",DSQRT(Q2)," # (SUSY SCALE)"
	WRITE(17,903) 3,3,HTAUS,"HTAU(Q,DR_bar)"

	WRITE(17,899) "# "
	WRITE(17,899) "# SOFT TRILINEAR COUPLINGS AT THE SUSY SCALE"
	WRITE(17,907) "BLOCK AU Q=",DSQRT(Q2)," # (SUSY SCALE)"
	WRITE(17,903) 3,3,PAR(12),"ATOP"
	WRITE(17,907) "BLOCK AD Q=",DSQRT(Q2)," # (SUSY SCALE)"
	WRITE(17,903) 3,3,PAR(13),"ABOT"
	WRITE(17,907) "BLOCK AE Q=",DSQRT(Q2)," # (SUSY SCALE)"
	WRITE(17,903) 2,2,PAR(24),"AMUON"
	WRITE(17,903) 3,3,PAR(14),"ATAU"

	WRITE(17,899) "# "
	WRITE(17,899) "# SOFT MASSES AT THE SUSY SCALE"
	WRITE(17,907) "BLOCK MSOFT Q=",DSQRT(Q2)," # (SUSY SCALE)"
	WRITE(17,902) 1,PAR(20),"M1"
	WRITE(17,902) 2,PAR(21),"M2"
	WRITE(17,902) 3,PAR(22),"M3"
	WRITE(17,902) 21,MHDS,"M_HD^2"
	WRITE(17,902) 22,MHUS,"M_HU^2"
	WRITE(17,902) 31,DSQRT(PAR(18)),"M_eL"
	WRITE(17,902) 32,DSQRT(PAR(18)),"M_muL"
	WRITE(17,902) 33,DSQRT(PAR(10)),"M_tauL"
	WRITE(17,902) 34,DSQRT(PAR(19)),"M_eR"
	WRITE(17,902) 35,DSQRT(PAR(19)),"M_muR"
	WRITE(17,902) 36,DSQRT(PAR(11)),"M_tauR"
	WRITE(17,902) 41,DSQRT(PAR(15)),"M_q1L"
	WRITE(17,902) 42,DSQRT(PAR(15)),"M_q2L"
	WRITE(17,902) 43,DSQRT(PAR(7)),"M_q3L"
	WRITE(17,902) 44,DSQRT(PAR(16)),"M_uR"
	WRITE(17,902) 45,DSQRT(PAR(16)),"M_cR"
	WRITE(17,902) 46,DSQRT(PAR(8)),"M_tR"
	WRITE(17,902) 47,DSQRT(PAR(17)),"M_dR"
	WRITE(17,902) 48,DSQRT(PAR(17)),"M_sR"
	WRITE(17,902) 49,DSQRT(PAR(9)),"M_bR"
	
	WRITE(17,899) "# "
	WRITE(17,899) "# NMSSM SPECIFIC PARAMETERS THE SUSY SCALE"
	WRITE(17,907) "BLOCK NMSSMRUN Q=",DSQRT(Q2)," # (SUSY SCALE)"
	WRITE(17,902) 1,PAR(1),"LAMBDA(Q,DR_bar)"
	WRITE(17,902) 2,PAR(2),"KAPPA(Q,DR_bar)"
	WRITE(17,902) 3,PAR(5),"ALAMBDA"
	WRITE(17,902) 4,PAR(6),"AKAPPA"
	WRITE(17,902) 5,MUQ,"LAMBDA*<S>=MU_EFF"
	WRITE(17,902) 10,MSSS,"M_S^2"
	
        IF(IFAIL.GT.10) GOTO 1

	WRITE(17,899) "# "
	WRITE(17,899) "# GAUGE AND YUKAWA COUPLINGS AT THE GUT SCALE"
	WRITE(17,907) "BLOCK GAUGE Q=MGUT=",MGUT," # (GUT SCALE)"
	WRITE(17,902) 1,DSQRT(5.D0/3.D0*G1GUT),
     C                "g1(MGUT,DR_bar), GUT normalization"
	WRITE(17,902) 2,DSQRT(G2GUT),"g2(MGUT,DR_bar)"
	WRITE(17,902) 3,DSQRT(G3GUT),"g3(MGUT,DR_bar)"
	WRITE(17,907) "BLOCK YU Q=MGUT=",MGUT," # (GUT SCALE)"
	WRITE(17,903) 3,3,HTOPGUT/DSQRT(DABS(HTOPGUT)),
     C	              "HTOP(MGUT,DR_bar)"
	WRITE(17,907) "BLOCK YD Q=MGUT=",MGUT," # (GUT SCALE)"
	WRITE(17,903) 3,3,HBOTGUT/DSQRT(DABS(HBOTGUT)),
     C	              "HBOT(MGUT,DR_bar)"
	WRITE(17,907) "BLOCK YE Q=MGUT=",MGUT," # (GUT SCALE)"
	WRITE(17,903) 3,3,HTAUGUT/DSQRT(DABS(HTAUGUT)),
     C                "HTAU(MGUT,DR_bar)"

	WRITE(17,899) "# "
	WRITE(17,899) "# SOFT TRILINEAR COUPLINGS AT THE GUT SCALE" 	
	WRITE(17,907) "BLOCK AU Q=MGUT=",MGUT," # (GUT SCALE)"
	WRITE(17,903) 3,3,ATGUT,"ATOP"
	WRITE(17,907) "BLOCK AD Q=MGUT=",MGUT," # (GUT SCALE)"
	WRITE(17,903) 3,3,ABGUT,"ABOT"
	WRITE(17,907) "BLOCK AE Q=MGUT=",MGUT," # (GUT SCALE)"
	WRITE(17,903) 2,2,AMUGUT,"AMUON"
	WRITE(17,903) 3,3,ATAUGUT,"ATAU"

	WRITE(17,899) "# "
	WRITE(17,899) "# SOFT MASSES SQUARED AT THE GUT SCALE"
	WRITE(17,907) "BLOCK MSOFT Q=MGUT=",MGUT," # (GUT SCALE)"
	WRITE(17,902) 1,M1GUT,"M1"
	WRITE(17,902) 2,M2GUT,"M2"
	WRITE(17,902) 3,M3GUT,"M3"
	WRITE(17,902) 21,MHDGUT,"M_HD^2"
	WRITE(17,902) 22,MHUGUT,"M_HU^2"
	WRITE(17,902) 31,MLGUT,"M_eL^2"
	WRITE(17,902) 32,MLGUT,"M_muL^2"
	WRITE(17,902) 33,ML3GUT,"M_tauL^2"
	WRITE(17,902) 34,MEGUT,"M_eR^2"
	WRITE(17,902) 35,MEGUT,"M_muR^2"
	WRITE(17,902) 36,ME3GUT,"M_tauR^2"
	WRITE(17,902) 41,MQGUT,"M_q1L^2"
	WRITE(17,902) 42,MQGUT,"M_q2L^2"
	WRITE(17,902) 43,MQ3GUT,"M_q3L^2"
	WRITE(17,902) 44,MUGUT,"M_uR^2"
	WRITE(17,902) 45,MUGUT,"M_cR^2"
	WRITE(17,902) 46,MU3GUT,"M_tR^2"
	WRITE(17,902) 47,MDGUT,"M_dR^2"
	WRITE(17,902) 48,MDGUT,"M_sR^2"
	WRITE(17,902) 49,MD3GUT,"M_bR^2"
#ifdef MAIN_MICROMEGAS
        WRITE(17,899) "BLOCK MO_HIGGS  #Higgs potential for MICROMEGAS"
        WRITE(17,902) 1, LQ/DSQRT(ZHU*ZHD*ZS),'Lambda'
        WRITE(17,902) 2, KQ/DSQRT(ZS**3),'Kappa'
        WRITE(17,902) 3, PAR(5),'aLmbd0'
        WRITE(17,902) 4, DELMB, 'deltaMb'
        WRITE(17,902) 5, 1/DSQRT(2.*dsqrt(2.D0)*GF) , 'vev'
        WRITE(17,899) '#Running masses'
        WRITE(17,902) 25, MH(1), 'lightest neutral scalar'
        WRITE(17,902) 35, MH(2), 'second neutral scalar'
        WRITE(17,902) 45, MH(3), 'third neutral scalar' 
        WRITE(17,902) 36, MA(1), 'lightest pseudoscalar'
        WRITE(17,902) 46, MA(2), 'second pseudoscalar'
        WRITE(17,902) 37, MHc  , 'charged Higgs'
#endif
	
	WRITE(17,899) "# "
	WRITE(17,899) "# NMSSM SPECIFIC PARAMETERS THE GUT SCALE"
	WRITE(17,907) "BLOCK NMSSMRUN Q=MGUT=",MGUT," # (GUT SCALE)"
	WRITE(17,902) 1,LGUT/DSQRT(DABS(LGUT)),"LAMBDA(MGUT,DR_bar)"
	WRITE(17,902) 2,KGUT/DSQRT(DABS(KGUT)),"KAPPA(MGUT,DR_bar)"
	WRITE(17,902) 3,ALGUT,"ALAMBDA"
	WRITE(17,902) 4,AKGUT,"AKAPPA"
	WRITE(17,902) 10,MSSGUT,"M_S^2"

 1	WRITE(18,899) "# HIGGS BRANCHING RATIOS IN SLHA FORMAT"
	WRITE(18,899) "# Info about decay package"
	WRITE(18,899) "BLOCK DCINFO        # Program information"
	WRITE(18,900) 1,"NMHDECAY   # Decay package"
	WRITE(18,900) 2,"3          # Version number"

	IF(IFAIL.GT.0 .AND. IFAIL.LT.10) GOTO 2

	WRITE(18,899) "#           PDG          Width"
	WRITE(18,904) 25,WIDTH(1),"Lightest neutral Higgs scalar"
	WRITE(18,905) BRJJ(1),2,21,21,"BR(H_1 -> gluon gluon)"
	WRITE(18,905) BRMM(1),2,13,-13,"BR(H_1 -> muon muon)"
	WRITE(18,905) BRLL(1),2,15,-15,"BR(H_1 -> tau tau)"
	WRITE(18,905) BRSS(1),2,3,-3,"BR(H_1 -> s sbar)"
	WRITE(18,905) BRCC(1),2,4,-4,"BR(H_1 -> c cbar)"
	WRITE(18,905) BRBB(1),2,5,-5,"BR(H_1 -> b bbar)"
	WRITE(18,905) BRTT(1),2,6,-6,"BR(H_1 -> t tbar)"
	WRITE(18,905) BRWW(1),2,24,-24,"BR(H_1 -> W+ W-)"
	WRITE(18,905) BRZZ(1),2,23,23,"BR(H_1 -> Z Z)"
	WRITE(18,905) BRGG(1),2,22,22,"BR(H_1 -> gamma gamma)"
	WRITE(18,905) BRZG(1),2,23,22,"BR(H_1 -> Z gamma)"
	IF(BRHAA(1,1).GT.0.D0)
     C  WRITE(18,905) BRHAA(1,1),2,36,36,"BR(H_1 -> A_1 A_1)"
        IF(BRHAA(1,2).GT.0.D0)
     C  WRITE(18,905) BRHAA(1,2),2,36,46,"BR(H_1 -> A_1 A_2)"
        IF(BRHAA(1,3).GT.0.D0)
     C  WRITE(18,905) BRHAA(1,3),2,46,46,"BR(H_1 -> A_2 A_2)"
        IF(BRHAZ(1,1).GT.0.D0)
     C  WRITE(18,905) BRHAZ(1,1),2,23,36,"BR(H_1 -> A_1 Z)"
        IF(BRNEU(1,1,1).GT.0.D0)
     C  WRITE(18,905) BRNEU(1,1,1),2,1000022,1000022,
     C		"BR(H_1 -> neu_1 neu_1)"
        IF(BRNEU(1,1,2).GT.0.D0)
     C  WRITE(18,905) BRNEU(1,1,2),2,1000022,1000023,
     C		"BR(H_1 -> neu_1 neu_2)"
        IF(BRNEU(1,1,3).GT.0.D0)
     C  WRITE(18,905) BRNEU(1,1,3),2,1000022,1000025,
     C		"BR(H_1 -> neu_1 neu_3)"
        IF(BRNEU(1,1,4).GT.0.D0)
     C  WRITE(18,905) BRNEU(1,1,4),2,1000022,1000035,
     C		"BR(H_1 -> neu_1 neu_4)"
        IF(BRNEU(1,1,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(1,1,5),2,1000022,1000045,
     C		"BR(H_1 -> neu_1 neu_5)"
        IF(BRNEU(1,2,2).GT.0.D0)
     C  WRITE(18,905) BRNEU(1,2,2),2,1000023,1000023,
     C		"BR(H_1 -> neu_2 neu_2)"
        IF(BRNEU(1,2,3).GT.0.D0)
     C  WRITE(18,905) BRNEU(1,2,3),2,1000023,1000025,
     C		"BR(H_1 -> neu_2 neu_3)"
        IF(BRNEU(1,2,4).GT.0.D0)
     C  WRITE(18,905) BRNEU(1,2,4),2,1000023,1000035,
     C		"BR(H_1 -> neu_2 neu_4)"
        IF(BRNEU(1,2,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(1,2,5),2,1000023,1000045,
     C		"BR(H_1 -> neu_2 neu_5)"
        IF(BRNEU(1,3,3).GT.0.D0)
     C  WRITE(18,905) BRNEU(1,3,3),2,1000025,1000025,
     C		"BR(H_1 -> neu_3 neu_3)"
        IF(BRNEU(1,3,4).GT.0.D0)
     C  WRITE(18,905) BRNEU(1,3,4),2,1000025,1000035,
     C		"BR(H_1 -> neu_3 neu_4)"
        IF(BRNEU(1,3,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(1,3,5),2,1000025,1000045,
     C		"BR(H_1 -> neu_3 neu_5)"
        IF(BRNEU(1,4,4).GT.0.D0)
     C  WRITE(18,905) BRNEU(1,4,4),2,1000035,1000035,
     C		"BR(H_1 -> neu_4 neu_4)"
        IF(BRNEU(1,4,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(1,4,5),2,1000035,1000045,
     C		"BR(H_1 -> neu_4 neu_5)"
        IF(BRNEU(1,5,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(1,5,5),2,1000045,1000045,
     C		"BR(H_1 -> neu_5 neu_5)"
        IF(BRCHA(1,1).GT.0.D0)
     C  WRITE(18,905) BRCHA(1,1),2,1000024,-1000024,
     C		"BR(H_1 -> cha_1 cha_1bar)"
        IF(BRCHA(1,2).GT.0.D0)
     C  WRITE(18,905) BRCHA(1,2),2,1000024,-1000037,
     C		"BR(H_1 -> cha_1 cha_2bar)"
        IF(BRCHA(1,2).GT.0.D0)
     C  WRITE(18,905) BRCHA(1,2),2,1000037,-1000024,
     C		"BR(H_1 -> cha_2 cha_1bar)"
        IF(BRCHA(1,3).GT.0.D0)
     C  WRITE(18,905) BRCHA(1,3),2,1000037,-1000037,
     C		"BR(H_1 -> cha_2 cha_2bar)"
        IF(BRHSQ(1,1).GT.0.D0)
     C  WRITE(18,905) BRHSQ(1,1),2,1000002,-1000002,
     C		"BR(H_1 -> ~u_L ~ubar_L)"
C======= ADDED FOR MICROMEGAS
        WRITE(17,899) "BLOCK MO_HIGGS  #Higgs potential for MICROMEGAS"
        WRITE(17,902) 1, LQ/DSQRT(ZHU*ZHD*ZS),'Lambda'
        WRITE(17,902) 2, KQ/DSQRT(ZS**3),'Kappa'
        WRITE(17,902) 3, PAR(5),'aLmbd0'
        WRITE(17,902) 4, DELMB, 'deltaMb'
        WRITE(17,902) 5, 1/DSQRT(2.*dsqrt(2.D0)*GF) , 'vev'
        WRITE(17,899) '#Running masses'
        WRITE(17,902) 25, MH(1), 'lightest neutral scalar'
        WRITE(17,902) 35, MH(2), 'second neutral scalar'
        WRITE(17,902) 45, MH(3), 'third neutral scalar' 
        WRITE(17,902) 36, MA(1), 'lightest pseudoscalar'
        WRITE(17,902) 46, MA(2), 'second pseudoscalar'
        WRITE(17,902) 37, MHc  , 'charged Higgs'

C================================
        IF(BRHSQ(1,1).GT.0.D0)
     C  WRITE(18,905) BRHSQ(1,1),2,1000004,-1000004,
     C		"BR(H_1 -> ~c_L ~cbar_L)"
        IF(BRHSQ(1,2).GT.0.D0)
     C  WRITE(18,905) BRHSQ(1,2),2,2000002,-2000002,
     C		"BR(H_1 -> ~u_R ~ubar_R)"
        IF(BRHSQ(1,2).GT.0.D0)
     C  WRITE(18,905) BRHSQ(1,2),2,2000004,-2000004,
     C		"BR(H_1 -> ~c_R ~cbar_R)"
        IF(BRHSQ(1,3).GT.0.D0)
     C  WRITE(18,905) BRHSQ(1,3),2,1000001,-1000001,
     C		"BR(H_1 -> ~d_L ~dbar_L)"
        IF(BRHSQ(1,3).GT.0.D0)
     C  WRITE(18,905) BRHSQ(1,3),2,1000003,-1000003,
     C		"BR(H_1 -> ~s_L ~sbar_L)"
        IF(BRHSQ(1,4).GT.0.D0)
     C  WRITE(18,905) BRHSQ(1,4),2,2000001,-2000001,
     C		"BR(H_1 -> ~d_R ~dbar_R)"
        IF(BRHSQ(1,4).GT.0.D0)
     C  WRITE(18,905) BRHSQ(1,4),2,2000003,-2000003,
     C		"BR(H_1 -> ~s_R ~sbar_R)"
        IF(BRHSQ(1,5).GT.0.D0)
     C  WRITE(18,905) BRHSQ(1,5),2,1000006,-1000006,
     C		"BR(H_1 -> ~t_1 ~tbar_1)"
        IF(BRHSQ(1,6).GT.0.D0)
     C  WRITE(18,905) BRHSQ(1,6),2,2000006,-2000006,
     C		"BR(H_1 -> ~t_2 ~tbar_2)"
        IF(BRHSQ(1,7).GT.0.D0)
     C  WRITE(18,905) BRHSQ(1,7),2,1000006,-2000006,
     C		"BR(H_1 -> ~t_1 ~tbar_2)"
        IF(BRHSQ(1,7).GT.0.D0)
     C  WRITE(18,905) BRHSQ(1,7),2,2000006,-1000006,
     C		"BR(H_1 -> ~t_2 ~tbar_1)"
        IF(BRHSQ(1,8).GT.0.D0)
     C  WRITE(18,905) BRHSQ(1,8),2,1000005,-1000005,
     C		"BR(H_1 -> ~b_1 ~bbar_1)"
        IF(BRHSQ(1,9).GT.0.D0)
     C  WRITE(18,905) BRHSQ(1,9),2,2000005,-2000005,
     C		"BR(H_1 -> ~b_2 ~bbar_2)"
        IF(BRHSQ(1,10).GT.0.D0)
     C  WRITE(18,905) BRHSQ(1,10),2,1000005,-2000005,
     C		"BR(H_1 -> ~b_1 ~bbar_2)"
        IF(BRHSQ(1,10).GT.0.D0)
     C  WRITE(18,905) BRHSQ(1,10),2,2000005,-1000005,
     C		"BR(H_1 -> ~b_2 ~bbar_1)"
        IF(BRHSL(1,1).GT.0.D0)
     C  WRITE(18,905) BRHSL(1,1),2,1000011,-1000011,
     C		"BR(H_1 -> ~e_L ~ebar_L)"
        IF(BRHSL(1,1).GT.0.D0)
     C  WRITE(18,905) BRHSL(1,1),2,1000013,-1000013,
     C		"BR(H_1 -> ~mu_L ~mubar_L)"
        IF(BRHSL(1,2).GT.0.D0)
     C  WRITE(18,905) BRHSL(1,2),2,2000011,-2000011,
     C		"BR(H_1 -> ~e_R ~ebar_R)"
        IF(BRHSL(1,2).GT.0.D0)
     C  WRITE(18,905) BRHSL(1,2),2,2000013,-2000013,
     C		"BR(H_1 -> ~mu_R ~mubarRL)"
        IF(BRHSL(1,3).GT.0.D0)
     C  WRITE(18,905) BRHSL(1,3),2,1000012,-1000012,
     C		"BR(H_1 -> ~nu_e_L ~nu_ebar_L)"
        IF(BRHSL(1,3).GT.0.D0)
     C  WRITE(18,905) BRHSL(1,3),2,1000014,-1000014,
     C		"BR(H_1 -> ~nu_mu_L ~nu_mubar_L)"
        IF(BRHSL(1,4).GT.0.D0)
     C  WRITE(18,905) BRHSL(1,4),2,1000015,-1000015,
     C		"BR(H_1 -> ~tau_1 ~taubar_1)"
        IF(BRHSL(1,5).GT.0.D0)
     C  WRITE(18,905) BRHSL(1,5),2,2000015,-2000015,
     C		"BR(H_1 -> ~tau_2 ~taubar_2)"
        IF(BRHSL(1,6).GT.0.D0)
     C  WRITE(18,905) BRHSL(1,6),2,1000015,-2000015,
     C		"BR(H_1 -> ~tau_1 ~taubar_2)"
        IF(BRHSL(1,6).GT.0.D0)
     C  WRITE(18,905) BRHSL(1,6),2,2000015,-1000015,
     C		"BR(H_1 -> ~tau_2 ~taubar_1)"
        IF(BRHSL(1,7).GT.0.D0)
     C  WRITE(18,905) BRHSL(1,7),2,1000016,-1000016,
     C		"BR(H_1 -> ~nu_tau_L ~nu_taubar_L)"
	
	WRITE(18,904) 35,WIDTH(2),"2nd neutral Higgs scalar"
	WRITE(18,905) BRJJ(2),2,21,21,"BR(H_2 -> gluon gluon)"
	WRITE(18,905) BRMM(2),2,13,-13,"BR(H_2 -> muon muon)"
	WRITE(18,905) BRLL(2),2,15,-15,"BR(H_2 -> tau tau)"
	WRITE(18,905) BRSS(2),2,3,-3,"BR(H_2 -> s sbar)"
	WRITE(18,905) BRCC(2),2,4,-4,"BR(H_2 -> c cbar)"
	WRITE(18,905) BRBB(2),2,5,-5,"BR(H_2 -> b bbar)"
	WRITE(18,905) BRTT(2),2,6,-6,"BR(H_2 -> t tbar)"
	WRITE(18,905) BRWW(2),2,24,-24,"BR(H_2 -> W+ W-)"
	WRITE(18,905) BRZZ(2),2,23,23,"BR(H_2 -> Z Z)"
	WRITE(18,905) BRGG(2),2,22,22,"BR(H_2 -> gamma gamma)"
	WRITE(18,905) BRZG(2),2,23,22,"BR(H_2 -> Z gamma)"
	IF(BRHHH(1).GT.0.D0)
     C  WRITE(18,905) BRHHH(1),2,25,25,"BR(H_2 -> H_1 H_1)"
	IF(BRHAA(2,1).GT.0.D0)
     C  WRITE(18,905) BRHAA(2,1),2,36,36,"BR(H_2 -> A_1 A_1)"
        IF(BRHAA(2,2).GT.0.D0)
     C  WRITE(18,905) BRHAA(2,2),2,36,46,"BR(H_2 -> A_1 A_2)"
        IF(BRHAA(2,3).GT.0.D0)
     C  WRITE(18,905) BRHAA(2,3),2,46,46,"BR(H_2 -> A_2 A_2)"
        IF(BRHAZ(2,1).GT.0.D0)
     C  WRITE(18,905) BRHAZ(2,1),2,23,36,"BR(H_2 -> A_1 Z)"
        IF(BRNEU(2,1,1).GT.0.D0)
     C  WRITE(18,905) BRNEU(2,1,1),2,1000022,1000022,
     C		"BR(H_2 -> neu_1 neu_1)"
        IF(BRNEU(2,1,2).GT.0.D0)
     C  WRITE(18,905) BRNEU(2,1,2),2,1000022,1000023,
     C		"BR(H_2 -> neu_1 neu_2)"
        IF(BRNEU(2,1,3).GT.0.D0)
     C  WRITE(18,905) BRNEU(2,1,3),2,1000022,1000025,
     C		"BR(H_2 -> neu_1 neu_3)"
        IF(BRNEU(2,1,4).GT.0.D0)
     C  WRITE(18,905) BRNEU(2,1,4),2,1000022,1000035,
     C		"BR(H_2 -> neu_1 neu_4)"
        IF(BRNEU(2,1,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(2,1,5),2,1000022,1000045,
     C		"BR(H_2 -> neu_1 neu_5)"
        IF(BRNEU(2,2,2).GT.0.D0)
     C  WRITE(18,905) BRNEU(2,2,2),2,1000023,1000023,
     C		"BR(H_2 -> neu_2 neu_2)"
        IF(BRNEU(2,2,3).GT.0.D0)
     C  WRITE(18,905) BRNEU(2,2,3),2,1000023,1000025,
     C		"BR(H_2 -> neu_2 neu_3)"
        IF(BRNEU(2,2,4).GT.0.D0)
     C  WRITE(18,905) BRNEU(2,2,4),2,1000023,1000035,
     C		"BR(H_2 -> neu_2 neu_4)"
        IF(BRNEU(2,2,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(2,2,5),2,1000023,1000045,
     C		"BR(H_2 -> neu_2 neu_5)"
        IF(BRNEU(2,3,3).GT.0.D0)
     C  WRITE(18,905) BRNEU(2,3,3),2,1000025,1000025,
     C		"BR(H_2 -> neu_3 neu_3)"
        IF(BRNEU(2,3,4).GT.0.D0)
     C  WRITE(18,905) BRNEU(2,3,4),2,1000025,1000035,
     C		"BR(H_2 -> neu_3 neu_4)"
        IF(BRNEU(2,3,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(2,3,5),2,1000025,1000045,
     C		"BR(H_2 -> neu_3 neu_5)"
        IF(BRNEU(2,4,4).GT.0.D0)
     C  WRITE(18,905) BRNEU(2,4,4),2,1000035,1000035,
     C		"BR(H_2 -> neu_4 neu_4)"
        IF(BRNEU(2,4,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(2,4,5),2,1000035,1000045,
     C		"BR(H_2 -> neu_4 neu_5)"
        IF(BRNEU(2,5,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(2,5,5),2,1000045,1000045,
     C		"BR(H_2 -> neu_5 neu_5)"
        IF(BRCHA(2,1).GT.0.D0)
     C  WRITE(18,905) BRCHA(2,1),2,1000024,-1000024,
     C		"BR(H_2 -> cha_1 cha_1bar)"
        IF(BRCHA(2,2).GT.0.D0)
     C  WRITE(18,905) BRCHA(2,2),2,1000024,-1000037,
     C		"BR(H_2 -> cha_1 cha_2bar)"
        IF(BRCHA(2,2).GT.0.D0)
     C  WRITE(18,905) BRCHA(2,2),2,1000037,-1000024,
     C		"BR(H_2 -> cha_2 cha_1bar)"
        IF(BRCHA(2,3).GT.0.D0)
     C  WRITE(18,905) BRCHA(2,3),2,1000037,-1000037,
     C		"BR(H_2 -> cha_2 cha_2bar)"
        IF(BRHSQ(2,1).GT.0.D0)
     C  WRITE(18,905) BRHSQ(2,1),2,1000002,-1000002,
     C		"BR(H_2 -> ~u_L ~ubar_L)"
        IF(BRHSQ(2,1).GT.0.D0)
     C  WRITE(18,905) BRHSQ(2,1),2,1000004,-1000004,
     C		"BR(H_2 -> ~c_L ~cbar_L)"
        IF(BRHSQ(2,2).GT.0.D0)
     C  WRITE(18,905) BRHSQ(2,2),2,2000002,-2000002,
     C		"BR(H_2 -> ~u_R ~ubar_R)"
        IF(BRHSQ(2,2).GT.0.D0)
     C  WRITE(18,905) BRHSQ(2,2),2,2000004,-2000004,
     C		"BR(H_2 -> ~c_R ~cbar_R)"
        IF(BRHSQ(2,3).GT.0.D0)
     C  WRITE(18,905) BRHSQ(2,3),2,1000001,-1000001,
     C		"BR(H_2 -> ~d_L ~dbar_L)"
        IF(BRHSQ(2,3).GT.0.D0)
     C  WRITE(18,905) BRHSQ(2,3),2,1000003,-1000003,
     C		"BR(H_2 -> ~s_L ~sbar_L)"
        IF(BRHSQ(2,4).GT.0.D0)
     C  WRITE(18,905) BRHSQ(2,4),2,2000001,-2000001,
     C		"BR(H_2 -> ~d_R ~dbar_R)"
        IF(BRHSQ(2,4).GT.0.D0)
     C  WRITE(18,905) BRHSQ(2,4),2,2000003,-2000003,
     C		"BR(H_2 -> ~s_R ~sbar_R)"
        IF(BRHSQ(2,5).GT.0.D0)
     C  WRITE(18,905) BRHSQ(2,5),2,1000006,-1000006,
     C		"BR(H_2 -> ~t_1 ~tbar_1)"
        IF(BRHSQ(2,6).GT.0.D0)
     C  WRITE(18,905) BRHSQ(2,6),2,2000006,-2000006,
     C		"BR(H_2 -> ~t_2 ~tbar_2)"
        IF(BRHSQ(2,7).GT.0.D0)
     C  WRITE(18,905) BRHSQ(2,7),2,1000006,-2000006,
     C		"BR(H_2 -> ~t_1 ~tbar_2)"
        IF(BRHSQ(2,7).GT.0.D0)
     C  WRITE(18,905) BRHSQ(2,7),2,2000006,-1000006,
     C		"BR(H_2 -> ~t_2 ~tbar_1)"
        IF(BRHSQ(2,8).GT.0.D0)
     C  WRITE(18,905) BRHSQ(2,8),2,1000005,-1000005,
     C		"BR(H_2 -> ~b_1 ~bbar_1)"
        IF(BRHSQ(2,9).GT.0.D0)
     C  WRITE(18,905) BRHSQ(2,9),2,2000005,-2000005,
     C		"BR(H_2 -> ~b_2 ~bbar_2)"
        IF(BRHSQ(2,10).GT.0.D0)
     C  WRITE(18,905) BRHSQ(2,10),2,1000005,-2000005,
     C		"BR(H_2 -> ~b_1 ~bbar_2)"
        IF(BRHSQ(2,10).GT.0.D0)
     C  WRITE(18,905) BRHSQ(2,10),2,2000005,-1000005,
     C		"BR(H_2 -> ~b_2 ~bbar_1)"
        IF(BRHSL(2,1).GT.0.D0)
     C  WRITE(18,905) BRHSL(2,1),2,1000011,-1000011,
     C		"BR(H_2 -> ~e_L ~ebar_L)"
        IF(BRHSL(2,1).GT.0.D0)
     C  WRITE(18,905) BRHSL(2,1),2,1000013,-1000013,
     C		"BR(H_2 -> ~mu_L ~mubar_L)"
        IF(BRHSL(2,2).GT.0.D0)
     C  WRITE(18,905) BRHSL(2,2),2,2000011,-2000011,
     C		"BR(H_2 -> ~e_R ~ebar_R)"
        IF(BRHSL(2,2).GT.0.D0)
     C  WRITE(18,905) BRHSL(2,2),2,2000013,-2000013,
     C		"BR(H_2 -> ~mu_R ~mubarRL)"
        IF(BRHSL(2,3).GT.0.D0)
     C  WRITE(18,905) BRHSL(2,3),2,1000012,-1000012,
     C		"BR(H_2 -> ~nu_e_L ~nu_ebar_L)"
        IF(BRHSL(2,3).GT.0.D0)
     C  WRITE(18,905) BRHSL(2,3),2,1000014,-1000014,
     C		"BR(H_2 -> ~nu_mu_L ~nu_mubar_L)"
        IF(BRHSL(2,4).GT.0.D0)
     C  WRITE(18,905) BRHSL(2,4),2,1000015,-1000015,
     C		"BR(H_2 -> ~tau_1 ~taubar_1)"
        IF(BRHSL(2,5).GT.0.D0)
     C  WRITE(18,905) BRHSL(2,5),2,2000015,-2000015,
     C		"BR(H_2 -> ~tau_2 ~taubar_2)"
        IF(BRHSL(2,6).GT.0.D0)
     C  WRITE(18,905) BRHSL(2,6),2,1000015,-2000015,
     C		"BR(H_2 -> ~tau_1 ~taubar_2)"
        IF(BRHSL(2,6).GT.0.D0)
     C  WRITE(18,905) BRHSL(2,6),2,2000015,-1000015,
     C		"BR(H_2 -> ~tau_2 ~taubar_1)"
        IF(BRHSL(2,7).GT.0.D0)
     C  WRITE(18,905) BRHSL(2,7),2,1000016,-1000016,
     C		"BR(H_2 -> ~nu_tau_L ~nu_taubar_L)"

	WRITE(18,904) 45,WIDTH(3),"3rd neutral Higgs scalar"
	WRITE(18,905) BRJJ(3),2,21,21,"BR(H_3 -> gluon gluon)"
	WRITE(18,905) BRMM(3),2,13,-13,"BR(H_3 -> muon muon)"
	WRITE(18,905) BRLL(3),2,15,-15,"BR(H_3 -> tau tau)"
	WRITE(18,905) BRSS(3),2,3,-3,"BR(H_3 -> s sbar)"
	WRITE(18,905) BRCC(3),2,4,-4,"BR(H_3 -> c cbar)"
	WRITE(18,905) BRBB(3),2,5,-5,"BR(H_3 -> b bbar)"
	WRITE(18,905) BRTT(3),2,6,-6,"BR(H_3 -> t tbar)"
	WRITE(18,905) BRWW(3),2,24,-24,"BR(H_3 -> W+ W-)"
	WRITE(18,905) BRZZ(3),2,23,23,"BR(H_3 -> Z Z)"
	WRITE(18,905) BRGG(3),2,22,22,"BR(H_3 -> gamma gamma)"
	WRITE(18,905) BRZG(3),2,23,22,"BR(H_3 -> Z gamma)"
	IF(BRHHH(2).GT.0.D0)
     C  WRITE(18,905) BRHHH(2),2,25,25,"BR(H_3 -> H_1 H_1)"
	IF(BRHHH(3).GT.0.D0)
     C  WRITE(18,905) BRHHH(3),2,25,35,"BR(H_3 -> H_1 H_2)"
	IF(BRHHH(4).GT.0.D0)
     C  WRITE(18,905) BRHHH(4),2,35,35,"BR(H_3 -> H_2 H_2)"
	IF(BRHAA(3,1).GT.0.D0)
     C  WRITE(18,905) BRHAA(3,1),2,36,36,"BR(H_3 -> A_1 A_1)"
        IF(BRHAA(3,2).GT.0.D0)
     C  WRITE(18,905) BRHAA(3,2),2,36,46,"BR(H_3 -> A_1 A_2)"
        IF(BRHAA(3,3).GT.0.D0)
     C  WRITE(18,905) BRHAA(3,3),2,46,46,"BR(H_3 -> A_2 A_2)"
        IF(BRHAZ(3,1).GT.0.D0)
     C  WRITE(18,905) BRHAZ(3,1),2,23,36,"BR(H_3 -> A_1 Z)"
        IF(BRHAZ(3,2).GT.0.D0)
     C  WRITE(18,905) BRHAZ(3,2),2,23,46,"BR(H_3 -> A_2 Z)"
        IF(BRHCHC(3).GT.0.D0)
     C  WRITE(18,905) BRHCHC(3),2,37,-37,"BR(H_3 -> H+ H-)"
        IF(BRHCW(3).GT.0.D0)
     C  WRITE(18,905) BRHCW(3),2,24,-37,"BR(H_3 -> W+ H-)"
        IF(BRNEU(3,1,1).GT.0.D0)
     C  WRITE(18,905) BRNEU(3,1,1),2,1000022,1000022,
     C		"BR(H_3 -> neu_1 neu_1)"
        IF(BRNEU(3,1,2).GT.0.D0)
     C  WRITE(18,905) BRNEU(3,1,2),2,1000022,1000023,
     C		"BR(H_3 -> neu_1 neu_2)"
        IF(BRNEU(3,1,3).GT.0.D0)
     C  WRITE(18,905) BRNEU(3,1,3),2,1000022,1000025,
     C		"BR(H_3 -> neu_1 neu_3)"
        IF(BRNEU(3,1,4).GT.0.D0)
     C  WRITE(18,905) BRNEU(3,1,4),2,1000022,1000035,
     C		"BR(H_3 -> neu_1 neu_4)"
        IF(BRNEU(3,1,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(3,1,5),2,1000022,1000045,
     C		"BR(H_3 -> neu_1 neu_5)"
        IF(BRNEU(3,2,2).GT.0.D0)
     C  WRITE(18,905) BRNEU(3,2,2),2,1000023,1000023,
     C		"BR(H_3 -> neu_2 neu_2)"
        IF(BRNEU(3,2,3).GT.0.D0)
     C  WRITE(18,905) BRNEU(3,2,3),2,1000023,1000025,
     C		"BR(H_3 -> neu_2 neu_3)"
        IF(BRNEU(3,2,4).GT.0.D0)
     C  WRITE(18,905) BRNEU(3,2,4),2,1000023,1000035,
     C		"BR(H_3 -> neu_2 neu_4)"
        IF(BRNEU(3,2,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(3,2,5),2,1000023,1000045,
     C		"BR(H_3 -> neu_2 neu_5)"
        IF(BRNEU(3,3,3).GT.0.D0)
     C  WRITE(18,905) BRNEU(3,3,3),2,1000025,1000025,
     C		"BR(H_3 -> neu_3 neu_3)"
        IF(BRNEU(3,3,4).GT.0.D0)
     C  WRITE(18,905) BRNEU(3,3,4),2,1000025,1000035,
     C		"BR(H_3 -> neu_3 neu_4)"
        IF(BRNEU(3,3,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(3,3,5),2,1000025,1000045,
     C		"BR(H_3 -> neu_3 neu_5)"
        IF(BRNEU(3,4,4).GT.0.D0)
     C  WRITE(18,905) BRNEU(3,4,4),2,1000035,1000035,
     C		"BR(H_3 -> neu_4 neu_4)"
        IF(BRNEU(3,4,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(3,4,5),2,1000035,1000045,
     C		"BR(H_3 -> neu_4 neu_5)"
        IF(BRNEU(3,5,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(3,5,5),2,1000045,1000045,
     C		"BR(H_3 -> neu_5 neu_5)"
        IF(BRCHA(3,1).GT.0.D0)
     C  WRITE(18,905) BRCHA(3,1),2,1000024,-1000024,
     C		"BR(H_3 -> cha_1 cha_1bar)"
        IF(BRCHA(3,2).GT.0.D0)
     C  WRITE(18,905) BRCHA(3,2),2,1000024,-1000037,
     C		"BR(H_3 -> cha_1 cha_2bar)"
        IF(BRCHA(3,2).GT.0.D0)
     C  WRITE(18,905) BRCHA(3,2),2,1000037,-1000024,
     C		"BR(H_3 -> cha_2 cha_1bar)"
        IF(BRCHA(3,3).GT.0.D0)
     C  WRITE(18,905) BRCHA(3,3),2,1000037,-1000037,
     C		"BR(H_3 -> cha_2 cha_2bar)"
        IF(BRHSQ(3,1).GT.0.D0)
     C  WRITE(18,905) BRHSQ(3,1),2,1000002,-1000002,
     C		"BR(H_3 -> ~u_L ~ubar_L)"
        IF(BRHSQ(3,1).GT.0.D0)
     C  WRITE(18,905) BRHSQ(3,1),2,1000004,-1000004,
     C		"BR(H_3 -> ~c_L ~cbar_L)"
        IF(BRHSQ(3,2).GT.0.D0)
     C  WRITE(18,905) BRHSQ(3,2),2,2000002,-2000002,
     C		"BR(H_3 -> ~u_R ~ubar_R)"
        IF(BRHSQ(3,2).GT.0.D0)
     C  WRITE(18,905) BRHSQ(3,2),2,2000004,-2000004,
     C		"BR(H_3 -> ~c_R ~cbar_R)"
        IF(BRHSQ(3,3).GT.0.D0)
     C  WRITE(18,905) BRHSQ(3,3),2,1000001,-1000001,
     C		"BR(H_3 -> ~d_L ~dbar_L)"
        IF(BRHSQ(3,3).GT.0.D0)
     C  WRITE(18,905) BRHSQ(3,3),2,1000003,-1000003,
     C		"BR(H_3 -> ~s_L ~sbar_L)"
        IF(BRHSQ(3,4).GT.0.D0)
     C  WRITE(18,905) BRHSQ(3,4),2,2000001,-2000001,
     C		"BR(H_3 -> ~d_R ~dbar_R)"
        IF(BRHSQ(3,4).GT.0.D0)
     C  WRITE(18,905) BRHSQ(3,4),2,2000003,-2000003,
     C		"BR(H_3 -> ~s_R ~sbar_R)"
        IF(BRHSQ(3,5).GT.0.D0)
     C  WRITE(18,905) BRHSQ(3,5),2,1000006,-1000006,
     C		"BR(H_3 -> ~t_1 ~tbar_1)"
        IF(BRHSQ(3,6).GT.0.D0)
     C  WRITE(18,905) BRHSQ(3,6),2,2000006,-2000006,
     C		"BR(H_3 -> ~t_2 ~tbar_2)"
        IF(BRHSQ(3,7).GT.0.D0)
     C  WRITE(18,905) BRHSQ(3,7),2,1000006,-2000006,
     C		"BR(H_3 -> ~t_1 ~tbar_2)"
        IF(BRHSQ(3,7).GT.0.D0)
     C  WRITE(18,905) BRHSQ(3,7),2,2000006,-1000006,
     C		"BR(H_3 -> ~t_2 ~tbar_1)"
        IF(BRHSQ(3,8).GT.0.D0)
     C  WRITE(18,905) BRHSQ(3,8),2,1000005,-1000005,
     C		"BR(H_3 -> ~b_1 ~bbar_1)"
        IF(BRHSQ(3,9).GT.0.D0)
     C  WRITE(18,905) BRHSQ(3,9),2,2000005,-2000005,
     C		"BR(H_3 -> ~b_2 ~bbar_2)"
        IF(BRHSQ(3,10).GT.0.D0)
     C  WRITE(18,905) BRHSQ(3,10),2,1000005,-2000005,
     C		"BR(H_3 -> ~b_1 ~bbar_2)"
        IF(BRHSQ(3,10).GT.0.D0)
     C  WRITE(18,905) BRHSQ(3,10),2,2000005,-1000005,
     C		"BR(H_3 -> ~b_2 ~bbar_1)"
        IF(BRHSL(3,1).GT.0.D0)
     C  WRITE(18,905) BRHSL(3,1),2,1000011,-1000011,
     C		"BR(H_3 -> ~e_L ~ebar_L)"
        IF(BRHSL(3,1).GT.0.D0)
     C  WRITE(18,905) BRHSL(3,1),2,1000013,-1000013,
     C		"BR(H_3 -> ~mu_L ~mubar_L)"
        IF(BRHSL(3,2).GT.0.D0)
     C  WRITE(18,905) BRHSL(3,2),2,2000011,-2000011,
     C		"BR(H_3 -> ~e_R ~ebar_R)"
        IF(BRHSL(3,2).GT.0.D0)
     C  WRITE(18,905) BRHSL(3,2),2,2000013,-2000013,
     C		"BR(H_3 -> ~mu_R ~mubarRL)"
        IF(BRHSL(3,3).GT.0.D0)
     C  WRITE(18,905) BRHSL(3,3),2,1000012,-1000012,
     C		"BR(H_3 -> ~nu_e_L ~nu_ebar_L)"
        IF(BRHSL(3,3).GT.0.D0)
     C  WRITE(18,905) BRHSL(3,3),2,1000014,-1000014,
     C		"BR(H_3 -> ~nu_mu_L ~nu_mubar_L)"
        IF(BRHSL(3,4).GT.0.D0)
     C  WRITE(18,905) BRHSL(3,4),2,1000015,-1000015,
     C		"BR(H_3 -> ~tau_1 ~taubar_1)"
        IF(BRHSL(3,5).GT.0.D0)
     C  WRITE(18,905) BRHSL(3,5),2,2000015,-2000015,
     C		"BR(H_3 -> ~tau_2 ~taubar_2)"
        IF(BRHSL(3,6).GT.0.D0)
     C  WRITE(18,905) BRHSL(3,6),2,1000015,-2000015,
     C		"BR(H_3 -> ~tau_1 ~taubar_2)"
        IF(BRHSL(3,6).GT.0.D0)
     C  WRITE(18,905) BRHSL(3,6),2,2000015,-1000015,
     C		"BR(H_3 -> ~tau_2 ~taubar_1)"
        IF(BRHSL(3,7).GT.0.D0)
     C  WRITE(18,905) BRHSL(3,7),2,1000016,-1000016,
     C		"BR(H_3 -> ~nu_tau_L ~nu_taubar_L)"

	WRITE(18,904) 36,WIDTH(4),"Lightest pseudoscalar"
	WRITE(18,905) BRJJ(4),2,21,21,"BR(A_1 -> gluon gluon)"
	WRITE(18,905) BRMM(4),2,13,-13,"BR(A_1 -> muon muon)"
	WRITE(18,905) BRLL(4),2,15,-15,"BR(A_1 -> tau tau)"
	WRITE(18,905) BRSS(4),2,3,-3,"BR(A_1 -> s sbar)"
	WRITE(18,905) BRCC(4),2,4,-4,"BR(A_1 -> c cbar)"
	WRITE(18,905) BRBB(4),2,5,-5,"BR(A_1 -> b bbar)"
	WRITE(18,905) BRTT(4),2,6,-6,"BR(A_1 -> t tbar)"
	WRITE(18,905) BRGG(4),2,22,22,"BR(A_1 -> gamma gamma)"
	WRITE(18,905) BRZG(4),2,23,22,"BR(A_1 -> Z gamma)"
	IF(BRAHZ(1,1).GT.0.D0)
     C  WRITE(18,905) BRAHZ(1,1),2,23,25,"BR(A_1 -> Z H_1)"
        IF(BRAHZ(1,2).GT.0.D0)
     C  WRITE(18,905) BRAHZ(1,2),2,23,35,"BR(A_1 -> Z H_2)"
        IF(BRNEU(4,1,1).GT.0.D0)
     C  WRITE(18,905) BRNEU(4,1,1),2,1000022,1000022,
     C		"BR(A_1 -> neu_1 neu_1)"
        IF(BRNEU(4,1,2).GT.0.D0)
     C  WRITE(18,905) BRNEU(4,1,2),2,1000022,1000023,
     C		"BR(A_1 -> neu_1 neu_2)"
        IF(BRNEU(4,1,3).GT.0.D0)
     C  WRITE(18,905) BRNEU(4,1,3),2,1000022,1000025,
     C		"BR(A_1 -> neu_1 neu_3)"
        IF(BRNEU(4,1,4).GT.0.D0)
     C  WRITE(18,905) BRNEU(4,1,4),2,1000022,1000035,
     C		"BR(A_1 -> neu_1 neu_4)"
        IF(BRNEU(4,1,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(4,1,5),2,1000022,1000045,
     C		"BR(A_1 -> neu_1 neu_5)"
        IF(BRNEU(4,2,2).GT.0.D0)
     C  WRITE(18,905) BRNEU(4,2,2),2,1000023,1000023,
     C		"BR(A_1 -> neu_2 neu_2)"
        IF(BRNEU(4,2,3).GT.0.D0)
     C  WRITE(18,905) BRNEU(4,2,3),2,1000023,1000025,
     C		"BR(A_1 -> neu_2 neu_3)"
        IF(BRNEU(4,2,4).GT.0.D0)
     C  WRITE(18,905) BRNEU(4,2,4),2,1000023,1000035,
     C		"BR(A_1 -> neu_2 neu_4)"
        IF(BRNEU(4,2,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(4,2,5),2,1000023,1000045,
     C		"BR(A_1 -> neu_2 neu_5)"
        IF(BRNEU(4,3,3).GT.0.D0)
     C  WRITE(18,905) BRNEU(4,3,3),2,1000025,1000025,
     C		"BR(A_1 -> neu_3 neu_3)"
        IF(BRNEU(4,3,4).GT.0.D0)
     C  WRITE(18,905) BRNEU(4,3,4),2,1000025,1000035,
     C		"BR(A_1 -> neu_3 neu_4)"
        IF(BRNEU(4,3,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(4,3,5),2,1000025,1000045,
     C		"BR(A_1 -> neu_3 neu_5)"
        IF(BRNEU(4,4,4).GT.0.D0)
     C  WRITE(18,905) BRNEU(4,4,4),2,1000035,1000035,
     C		"BR(A_1 -> neu_4 neu_4)"
        IF(BRNEU(4,4,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(4,4,5),2,1000035,1000045,
     C		"BR(A_1 -> neu_4 neu_5)"
        IF(BRNEU(4,5,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(4,5,5),2,1000045,1000045,
     C		"BR(A_1 -> neu_5 neu_5)"
        IF(BRCHA(4,1).GT.0.D0)
     C  WRITE(18,905) BRCHA(4,1),2,1000024,-1000024,
     C		"BR(A_1 -> cha_1 cha_1bar)"
        IF(BRCHA(4,2).GT.0.D0)
     C  WRITE(18,905) BRCHA(4,2),2,1000024,-1000037,
     C		"BR(A_1 -> cha_1 cha_2bar)"
        IF(BRCHA(4,2).GT.0.D0)
     C  WRITE(18,905) BRCHA(4,2),2,1000037,-1000024,
     C		"BR(A_1 -> cha_2 cha_1bar)"
        IF(BRCHA(4,3).GT.0.D0)
     C  WRITE(18,905) BRCHA(4,3),2,1000037,-1000037,
     C		"BR(A_1 -> cha_2 cha_2bar)"
        IF(BRASQ(1,1).GT.0.D0)
     C  WRITE(18,905) BRASQ(1,1),2,1000006,-1000006,
     C		"BR(A_1 -> ~t_1 ~tbar_1)"
        IF(BRASQ(1,2).GT.0.D0)
     C  WRITE(18,905) BRASQ(1,2),2,2000006,-2000006,
     C		"BR(A_1 -> ~t_2 ~tbar_2)"
        IF(BRASQ(1,3).GT.0.D0)
     C  WRITE(18,905) BRASQ(1,3),2,1000006,-2000006,
     C		"BR(A_1 -> ~t_1 ~tbar_2)"
        IF(BRASQ(1,3).GT.0.D0)
     C  WRITE(18,905) BRASQ(1,3),2,2000006,-1000006,
     C		"BR(A_1 -> ~t_2 ~tbar_1)"
        IF(BRASQ(1,4).GT.0.D0)
     C  WRITE(18,905) BRASQ(1,4),2,1000005,-1000005,
     C		"BR(A_1 -> ~b_1 ~bbar_1)"
        IF(BRASQ(1,5).GT.0.D0)
     C  WRITE(18,905) BRASQ(1,5),2,2000005,-2000005,
     C		"BR(A_1 -> ~b_2 ~bbar_2)"
        IF(BRASQ(1,6).GT.0.D0)
     C  WRITE(18,905) BRASQ(1,6),2,1000005,-2000005,
     C		"BR(A_1 -> ~b_1 ~bbar_2)"
        IF(BRASQ(1,6).GT.0.D0)
     C  WRITE(18,905) BRASQ(1,6),2,2000005,-1000005,
     C		"BR(A_1 -> ~b_2 ~bbar_1)"
        IF(BRASL(1,1).GT.0.D0)
     C  WRITE(18,905) BRASL(1,1),2,1000015,-1000015,
     C		"BR(A_1 -> ~tau_1 ~taubar_1)"
        IF(BRASL(1,2).GT.0.D0)
     C  WRITE(18,905) BRASL(1,2),2,2000015,-2000015,
     C		"BR(A_1 -> ~tau_2 ~taubar_2)"
        IF(BRASL(1,3).GT.0.D0)
     C  WRITE(18,905) BRASL(1,3),2,1000015,-2000015,
     C		"BR(A_1 -> ~tau_1 ~taubar_2)"
        IF(BRASL(1,3).GT.0.D0)
     C  WRITE(18,905) BRASL(1,3),2,2000015,-1000015,
     C		"BR(A_1 -> ~tau_2 ~taubar_1)"

	WRITE(18,904) 46,WIDTH(5),"2nd pseudoscalar"
	WRITE(18,905) BRJJ(5),2,21,21,"BR(A_2 -> gluon gluon)"
	WRITE(18,905) BRMM(5),2,13,-13,"BR(A_2 -> muon muon)"
	WRITE(18,905) BRLL(5),2,15,-15,"BR(A_2 -> tau tau)"
	WRITE(18,905) BRSS(5),2,3,-3,"BR(A_2 -> s sbar)"
	WRITE(18,905) BRCC(5),2,4,-4,"BR(A_2 -> c cbar)"
	WRITE(18,905) BRBB(5),2,5,-5,"BR(A_2 -> b bbar)"
	WRITE(18,905) BRTT(5),2,6,-6,"BR(A_2 -> t tbar)"
	WRITE(18,905) BRGG(5),2,22,22,"BR(A_2 -> gamma gamma)"
	WRITE(18,905) BRZG(5),2,23,22,"BR(A_2 -> Z gamma)"
	IF(BRAHA(1).GT.0.D0)
     C  WRITE(18,905) BRAHA(1),2,36,25,"BR(A_2 -> A_1 H_1)"
        IF(BRAHA(2).GT.0.D0)
     C  WRITE(18,905) BRAHA(2),2,36,35,"BR(A_2 -> A_1 H_2)"
	IF(BRAHA(3).GT.0.D0)
     C  WRITE(18,905) BRAHA(3),2,36,45,"BR(A_2 -> A_1 H_3)"
	IF(BRAHZ(2,1).GT.0.D0)
     C  WRITE(18,905) BRAHZ(2,1),2,23,25,"BR(A_2 -> Z H_1)"
	IF(BRAHZ(2,2).GT.0.D0)
     C  WRITE(18,905) BRAHZ(2,2),2,23,35,"BR(A_2 -> Z H_2)"
	IF(BRHCW(5).GT.0.D0)
     C  WRITE(18,905) BRHCW(5),2,24,-37,"BR(A_2 -> W+ H-)"
        IF(BRNEU(5,1,1).GT.0.D0)
     C  WRITE(18,905) BRNEU(5,1,1),2,1000022,1000022,
     C		"BR(A_2 -> neu_1 neu_1)"
        IF(BRNEU(5,1,2).GT.0.D0)
     C  WRITE(18,905) BRNEU(5,1,2),2,1000022,1000023,
     C		"BR(A_2 -> neu_1 neu_2)"
        IF(BRNEU(5,1,3).GT.0.D0)
     C  WRITE(18,905) BRNEU(5,1,3),2,1000022,1000025,
     C		"BR(A_2 -> neu_1 neu_3)"
        IF(BRNEU(5,1,4).GT.0.D0)
     C  WRITE(18,905) BRNEU(5,1,4),2,1000022,1000035,
     C		"BR(A_2 -> neu_1 neu_4)"
        IF(BRNEU(5,1,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(5,1,5),2,1000022,1000045,
     C		"BR(A_2 -> neu_1 neu_5)"
        IF(BRNEU(5,2,2).GT.0.D0)
     C  WRITE(18,905) BRNEU(5,2,2),2,1000023,1000023,
     C		"BR(A_2 -> neu_2 neu_2)"
        IF(BRNEU(5,2,3).GT.0.D0)
     C  WRITE(18,905) BRNEU(5,2,3),2,1000023,1000025,
     C		"BR(A_2 -> neu_2 neu_3)"
        IF(BRNEU(5,2,4).GT.0.D0)
     C  WRITE(18,905) BRNEU(5,2,4),2,1000023,1000035,
     C		"BR(A_2 -> neu_2 neu_4)"
        IF(BRNEU(5,2,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(5,2,5),2,1000023,1000045,
     C		"BR(A_2 -> neu_2 neu_5)"
        IF(BRNEU(5,3,3).GT.0.D0)
     C  WRITE(18,905) BRNEU(5,3,3),2,1000025,1000025,
     C		"BR(A_2 -> neu_3 neu_3)"
        IF(BRNEU(5,3,4).GT.0.D0)
     C  WRITE(18,905) BRNEU(5,3,4),2,1000025,1000035,
     C		"BR(A_2 -> neu_3 neu_4)"
        IF(BRNEU(5,3,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(5,3,5),2,1000025,1000045,
     C		"BR(A_2 -> neu_3 neu_5)"
        IF(BRNEU(5,4,4).GT.0.D0)
     C  WRITE(18,905) BRNEU(5,4,4),2,1000035,1000035,
     C		"BR(A_2 -> neu_4 neu_4)"
        IF(BRNEU(5,4,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(5,4,5),2,1000035,1000045,
     C		"BR(A_2 -> neu_4 neu_5)"
        IF(BRNEU(5,5,5).GT.0.D0)
     C  WRITE(18,905) BRNEU(5,5,5),2,1000045,1000045,
     C		"BR(A_2 -> neu_5 neu_5)"
        IF(BRCHA(5,1).GT.0.D0)
     C  WRITE(18,905) BRCHA(5,1),2,1000024,-1000024,
     C		"BR(A_2 -> cha_1 cha_1bar)"
        IF(BRCHA(5,2).GT.0.D0)
     C  WRITE(18,905) BRCHA(5,2),2,1000024,-1000037,
     C		"BR(A_2 -> cha_1 cha_2bar)"
        IF(BRCHA(5,2).GT.0.D0)
     C  WRITE(18,905) BRCHA(5,2),2,1000037,-1000024,
     C		"BR(A_2 -> cha_2 cha_1bar)"
        IF(BRCHA(5,3).GT.0.D0)
     C  WRITE(18,905) BRCHA(5,3),2,1000037,-1000037,
     C		"BR(A_2 -> cha_2 cha_2bar)"
        IF(BRASQ(2,1).GT.0.D0)
     C  WRITE(18,905) BRASQ(2,1),2,1000006,-1000006,
     C		"BR(A_2 -> ~t_1 ~tbar_1)"
        IF(BRASQ(2,2).GT.0.D0)
     C  WRITE(18,905) BRASQ(2,2),2,2000006,-2000006,
     C		"BR(A_2 -> ~t_2 ~tbar_2)"
        IF(BRASQ(2,3).GT.0.D0)
     C  WRITE(18,905) BRASQ(2,3),2,1000006,-2000006,
     C		"BR(A_2 -> ~t_1 ~tbar_2)"
        IF(BRASQ(2,3).GT.0.D0)
     C  WRITE(18,905) BRASQ(2,3),2,2000006,-1000006,
     C		"BR(A_2 -> ~t_2 ~tbar_1)"
        IF(BRASQ(2,4).GT.0.D0)
     C  WRITE(18,905) BRASQ(2,4),2,1000005,-1000005,
     C		"BR(A_2 -> ~b_1 ~bbar_1)"
        IF(BRASQ(2,5).GT.0.D0)
     C  WRITE(18,905) BRASQ(2,5),2,2000005,-2000005,
     C		"BR(A_2 -> ~b_2 ~bbar_2)"
        IF(BRASQ(2,6).GT.0.D0)
     C  WRITE(18,905) BRASQ(2,6),2,1000005,-2000005,
     C		"BR(A_2 -> ~b_1 ~bbar_2)"
        IF(BRASQ(2,6).GT.0.D0)
     C  WRITE(18,905) BRASQ(2,6),2,2000005,-1000005,
     C		"BR(A_2 -> ~b_2 ~bbar_1)"
        IF(BRASL(2,1).GT.0.D0)
     C  WRITE(18,905) BRASL(2,1),2,1000015,-1000015,
     C		"BR(A_2 -> ~tau_1 ~taubar_1)"
        IF(BRASL(2,2).GT.0.D0)
     C  WRITE(18,905) BRASL(2,2),2,2000015,-2000015,
     C		"BR(A_2 -> ~tau_2 ~taubar_2)"
        IF(BRASL(2,3).GT.0.D0)
     C  WRITE(18,905) BRASL(2,3),2,1000015,-2000015,
     C		"BR(A_2 -> ~tau_1 ~taubar_2)"
        IF(BRASL(2,3).GT.0.D0)
     C  WRITE(18,905) BRASL(2,3),2,2000015,-1000015,
     C		"BR(A_2 -> ~tau_2 ~taubar_1)"

	WRITE(18,904) 37,HCWIDTH,"Charged Higgs"
	WRITE(18,905) HCBRM,2,-13,14,"BR(H+ -> muon nu_muon)"
	WRITE(18,905) HCBRL,2,-15,16,"BR(H+ -> tau nu_tau)"
	WRITE(18,905) HCBRSU,2,2,-3,"BR(H+ -> u sbar)"
	WRITE(18,905) HCBRSC,2,4,-3,"BR(H+ -> c sbar)"
	WRITE(18,905) HCBRBU,2,2,-5,"BR(H+ -> u bbar)"
	WRITE(18,905) HCBRBC,2,4,-5,"BR(H+ -> c bbar)"
	WRITE(18,905) HCBRBT,2,6,-5,"BR(H+ -> t bbar)"
	IF(HCBRWH(1).GT.0.D0)
     C  WRITE(18,905) HCBRWH(1),2,24,25,"BR(H+ -> W+ H_1)"
        IF(HCBRWH(2).GT.0.D0)
     C  WRITE(18,905) HCBRWH(2),2,24,35,"BR(H+ -> W+ H_2)"
        IF(HCBRWH(4).GT.0.D0)
     C  WRITE(18,905) HCBRWH(4),2,24,36,"BR(H+ -> W+ A_1)"
        IF(HCBRNC(1,1).GT.0.D0)
     C  WRITE(18,905) HCBRNC(1,1),2,1000024,1000022,
     C		"BR(H+ -> cha_1 neu_1)"
        IF(HCBRNC(2,1).GT.0.D0)
     C  WRITE(18,905) HCBRNC(2,1),2,1000024,1000023,
     C		"BR(H+ -> cha_1 neu_2)"
        IF(HCBRNC(3,1).GT.0.D0)
     C  WRITE(18,905) HCBRNC(3,1),2,1000024,1000025,
     C		"BR(H+ -> cha_1 neu_3)"
        IF(HCBRNC(4,1).GT.0.D0)
     C  WRITE(18,905) HCBRNC(4,1),2,1000024,1000035,
     C		"BR(H+ -> cha_1 neu_4)"
        IF(HCBRNC(5,1).GT.0.D0)
     C  WRITE(18,905) HCBRNC(5,1),2,1000024,1000045,
     C		"BR(H+ -> cha_1 neu_5)"
        IF(HCBRNC(1,2).GT.0.D0)
     C  WRITE(18,905) HCBRNC(1,2),2,1000037,1000022,
     C		"BR(H+ -> cha_2 neu_1)"
        IF(HCBRNC(2,2).GT.0.D0)
     C  WRITE(18,905) HCBRNC(2,2),2,1000037,1000023,
     C		"BR(H+ -> cha_2 neu_2)"
        IF(HCBRNC(3,2).GT.0.D0)
     C  WRITE(18,905) HCBRNC(3,2),2,1000037,1000025,
     C		"BR(H+ -> cha_2 neu_3)"
        IF(HCBRNC(4,2).GT.0.D0)
     C  WRITE(18,905) HCBRNC(4,2),2,1000037,1000035,
     C		"BR(H+ -> cha_2 neu_4)"
        IF(HCBRNC(5,2).GT.0.D0)
     C  WRITE(18,905) HCBRNC(5,2),2,1000037,1000045,
     C		"BR(H+ -> cha_2 neu_5)"
        IF(HCBRSQ(1).GT.0.D0)
     C  WRITE(18,905) HCBRSQ(1),2,1000002,-1000001,
     C		"BR(H+ -> ~u_L ~dbar_L)"
        IF(HCBRSQ(1).GT.0.D0)
     C  WRITE(18,905) HCBRSQ(1),2,1000004,-1000003,
     C		"BR(H+ -> ~c_L ~sbar_L)"
        IF(HCBRSQ(2).GT.0.D0)
     C  WRITE(18,905) HCBRSQ(2),2,1000006,-1000005,
     C		"BR(H+ -> ~t_1 ~bbar_1)"
        IF(HCBRSQ(3).GT.0.D0)
     C  WRITE(18,905) HCBRSQ(3),2,1000006,-2000005,
     C		"BR(H+ -> ~t_1 ~bbar_2)"
        IF(HCBRSQ(4).GT.0.D0)
     C  WRITE(18,905) HCBRSQ(4),2,2000006,-1000005,
     C		"BR(H+ -> ~t_2 ~bbar_1)"
        IF(HCBRSQ(5).GT.0.D0)
     C  WRITE(18,905) HCBRSQ(5),2,2000006,-2000005,
     C		"BR(H+ -> ~t_2 ~bbar_2)"
        IF(HCBRSL(1).GT.0.D0)
     C  WRITE(18,905) HCBRSL(1),2,1000012,-1000011,
     C		"BR(H+ -> ~nu_e_L ~ebar_L)"
        IF(HCBRSL(1).GT.0.D0)
     C  WRITE(18,905) HCBRSL(1),2,1000014,-1000013,
     C		"BR(H+ -> ~nu_mu_L ~mubar_L)"
        IF(HCBRSL(2).GT.0.D0)
     C  WRITE(18,905) HCBRSL(2),2,1000016,-1000015,
     C		"BR(H+ -> ~nu_tau_L ~taubar_1)"
        IF(HCBRSL(3).GT.0.D0)
     C  WRITE(18,905) HCBRSL(3),2,1000016,-2000015,
     C		"BR(H+ -> ~nu_tau_L ~taubar_2)"

 2	WRITE(19,899) "# RELIC DENSITY CALCULATED BY MICROMEGAS"
	WRITE(19,899) "#"
	WRITE(19,899) "BLOCK RDINFO        # Program information"
	WRITE(19,900) 1,"MicrOmegas # Dark matter package"
	WRITE(19,900) 2,"2.0        # Version number"

	IF(IFAIL.EQ.0 .OR. IFAIL.GE.10)THEN
#ifdef  MAIN_NMSSMTOOLS
	 IF(OMGFLAG.NE.0)THEN

	  IF(OMG.GT.0.D0)THEN
           IF(PROB(30).GT.0.D0)WRITE(19,900) 3,
     C      "# Relic density too large (WMAP)"
           IF(PROB(30).LT.0.D0)WRITE(19,900) 3,
     C      "# Relic density too small (WMAP)"
           IF(PROB(31).NE.0.D0)WRITE(19,900) 4,
     C      "# Unreliable Higgs widths in micrOMEGAs"
	   WRITE(19,899) "#"
	   WRITE(19,899) "BLOCK RELDEN"
	   WRITE(19,910) OMG,"# omega h^2"
	   WRITE(19,910) Xf,"# Xf"
	   WRITE(19,899) "#"
	   WRITE(19,899) "BLOCK LSP"
	   WRITE(19,910) LOPMASS(),"# LSP mass"
	   CALL O1CONTENTS(19)
	   WRITE(19,899) "#"
	   WRITE(19,899) "BLOCK CHANNELS"
	   CALL PRINTCHANNELS(Xf,1.D-3,1.D-4,1,19)
	   IF(OMGFLAG.EQ.2)THEN
	    WRITE(19,899) "#"
	    WRITE(19,899) "BLOCK SPECTRUM"
	    WRITE(19,910) sigmaV,"# sigmaV"
	    WRITE(19,909) "x","dN/dx"
	    DO I=1,NBIN
	     WRITE(19,908) x(I),dNdx(I)
	    ENDDO
	   ENDIF
	  ELSEIF(OMG.EQ.0.D0)THEN
	   WRITE(19,900) 4,"# Cannot compute Omega h^2 (mLSP < 1 GeV)"
	  ELSEIF(OMG.EQ.-1.D0)THEN
	   WRITE(19,900) 4,"# Lightest neutralino is not the LSP"
	  ENDIF

	 ELSE

	  WRITE(19,900) 4,"# Omega h^2 not computed (OMGFLAG=0)"
	
	 ENDIF
#endif
	ELSE

	 WRITE(19,900) 4,"# Cannot compute Omega h^2 (0<IFAIL<10)"

	ENDIF

 899    FORMAT(A)
 900    FORMAT(1X,I5,3X,A)
 901    FORMAT(1X,I5,3X,1P,E16.8,0P,3X,'#',1X,A)
 902    FORMAT(1X,I9,3X,1P,E16.8,0P,3X,'#',1X,A)
 903    FORMAT(1X,I2,1X,I2,3X,1P,E16.8,0P,3X,'#',1X,A)
 904    FORMAT('DECAY',1X,I9,3X,1P,E16.8,0P,3X,'#',1X,A)
 905    FORMAT(3X,1P,E16.8,0P,3X,I2,3X,I9,1X,I9,1X,2X,'#',1X,A)
 906    FORMAT('#',1X,A,3X,E16.8)
 907    FORMAT(A,1P,E16.8,A)
 908	FORMAT(2E16.8)
 909	FORMAT(8X,A,12X,A)
 910    FORMAT(E16.8,3X,A)
 911    FORMAT(A,F6.3,A,F6.3,A)

	END
